/* 	TicketBroker.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 26-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		rewrote close operation, ending tasks gave too much trouble
 * Mar 28, 2008	dennis@brains2b.org		close() might finish before everything was saved
 * TODO document this class		
*/
package org.brains2b.coffee;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.TimerTask;

import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.file.FileConnector;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

/**
 * [Short description] [Long description]
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl </A>
 * @version [version] [lastrevisiondate]
 */
public class TicketBroker extends TimerTask implements TaskListener {
    private final static TicketBroker m_instance = new TicketBroker();
    private final static int MAX=5;
    private static final int[] SEQUENCE = new int[] {
        CoffeeEvent.NEWS_GROUP,CoffeeEvent.GET_ARTICLES,1,
        CoffeeEvent.SAVE_ATTACHMENT,CoffeeEvent.SAVE,2,
        CoffeeEvent.GET_ARTICLES,CoffeeEvent.XML_SAVE,1,
        CoffeeEvent.GET_GROUPLIST,CoffeeEvent.XML_SAVE,1,
        CoffeeEvent.ENCODE_POST, CoffeeEvent.POST_ARTICLE,1,
        CoffeeEvent.SAVE_POST, CoffeeEvent.XML_SAVE,1
    };
    
    private boolean m_closing=false;
    private int m_tasks=0;
    private List m_listeners;
	private TicketQueue m_queue;
    private HashMap m_actions;
    private HashMap m_servers;
    
    protected TicketBroker() {
		super();
		m_queue=new TicketQueue();
		m_actions = new HashMap();
		m_servers = new HashMap();
        registerHandler(m_queue);
		addTaskListener(this);
    }
    
    public final static TicketBroker getBroker() {
        return m_instance;
    }

    public void broker(CoffeeEvent event) {
        
        if (event.getID()==CoffeeEvent.PROCESS) {
            CoffeeExchangeControl cec= (CoffeeExchangeControl) event.getArgument();
            Thread t = new Thread(cec);
        	t.start();
        } else if (event.getID()==CoffeeEvent.CHANGE_SERVER) {
            ServerDescriptor sd= (ServerDescriptor) event.getArgument();
            registerHandler(getServer(sd));
            registerHandler(new FileConnector(sd.getId(),new File(sd.getPath())));
        } else if (event.getID()==CoffeeEvent.EXPIRE_SERVER) {
        	ServerDescriptor sd= (ServerDescriptor) event.getArgument();
        	NewsServer current = (NewsServer) getTicketHandler(CoffeeEvent.GET_ARTICLE);
        	NewsServer expire =  (NewsServer) m_servers.remove(sd);
			if (expire==current) {
        		broker(new CoffeeEvent(event.getExchange(),CoffeeEvent.CHANGE_SERVER,sd));
        	}
        } else {
            TicketHandler ticketHandler= getTicketHandler(event.getID());
            Ticket ticket = new Ticket(ticketHandler, event);
            if (event.getID()>=CoffeeEvent.QUEUE_REMOVE && event.getID()<=CoffeeEvent.QUEUE_RESUME) {
            	startTicket(ticket);
            } else if (m_tasks<MAX && ticketHandler.isAvailable() && m_queue.isEmpty()) {
                startTicket(ticket);
            } else {
               addToQueue(ticket);
            }
        }
        showTasks();

    }
    
    public void close() throws Exception {
        
    	m_closing =true;
        m_queue.clear();
        List handlers= getHandlers();
        for (Iterator it =handlers.iterator();it.hasNext();) {
           ((TicketHandler) it.next()).close();
       	}
        m_closing = false;
        while (m_tasks>0) {
           try {wait(500);} catch (Exception ex){}
       }
    }
    
    private List getHandlers() {
        ArrayList handlers=new ArrayList();
        for (Iterator it= m_actions.values().iterator();it.hasNext();) {
            TicketHandler h = (TicketHandler) it.next();
            if (!handlers.contains(h)) {
                handlers.add(h);
            }
        }
        for (Iterator it= m_servers.values().iterator();it.hasNext();) {
            TicketHandler h = (TicketHandler) it.next();
            if (!handlers.contains(h)) {
                handlers.add(h);
            }
        }
        return handlers;
    }
    
    private void startTicket(Ticket ticket) {
    	final int eventId = ticket.getEvent().getID();
        if (m_closing && eventId!=CoffeeEvent.XML_SAVE && eventId!=CoffeeEvent.XML_SAVE_CONFIG && eventId!=CoffeeEvent.SAVE) return;
        if (ticket.getEvent().getSource() instanceof CoffeeExchangeControl) {
            CoffeeExchangeControl cec= (CoffeeExchangeControl) ticket.getEvent().getSource();
            ticket.addTaskListener(cec.getTaskListener());
            ticket.addTaskListener(this);
        } else {
            if (m_listeners != null) {
                for (int i = 0; i < m_listeners.size(); i++) {
                    ticket.addTaskListener((TaskListener) m_listeners.get(i));
                }
            }
       }
        int seq=getSequence(ticket.getEvent().getID());
        boolean callBack = isCallBack(ticket.getEvent().getID());
        if (seq!=-1 || callBack) {
            ticket.addTaskListener(new TicketFarm(seq,callBack));
        }
            
        m_tasks++;
        showTasks();
        Thread t = new Thread(ticket);
        t.start();
    }
    
    public void run() {
        if (!m_queue.isEmpty()) {
            if (nextTask()) {
                showTasks();
            }
        }
    }
    
    public void addTaskListener(TaskListener listener) {
        if (m_listeners == null) {
            m_listeners = new ArrayList();
        }
        m_listeners.add(listener);
    }

    public void removeTaskListener(TaskListener listener) {
        if (m_listeners != null) {
            m_listeners.remove(listener);
        }
    }

    public void taskProgressed(TaskEvent e) {
        if (e.isFinished()) {
            if (e.getSource() instanceof Ticket) {
                Ticket t= (Ticket) e.getSource();
                CoffeeEvent ce= t.getEvent();
                if (ce.getExchange()!=null && t.getMessage()!=null && t.getMessage().getType()>=0) {
                    ce.getExchange().setCurrentMessage(t.getMessage());
                }
            }
            m_tasks--;
            showTasks();         
        }
        
    }
    
    private synchronized boolean nextTask() {
        Iterator it=m_queue.iterator();
        synchronized (it) {
	        boolean b=false;
	        while(it.hasNext() && m_tasks<MAX) {
	            Ticket t=(Ticket) it.next();
	            if (t.getTicketHandler().isAvailable() && !t.isWait()) {
	               if (!t.isFinished() && t.getResult()!=Ticket.RESULT_REMOVE) {
	            		startTicket(t);
	               }
	               it.remove();
	               b=true;
	            }
	        }
	        return b;
        }
        
    }
    
    private int getQueueSize() {
        return m_tasks+m_queue.getSize();   
    }
    
    private int getRunTasks() {
        return m_tasks;
    }
    
    private void showTasks() {
        TaskEvent e=new TaskEvent(this,-1,0,getRunTasks(),getQueueSize());
        e.setFinished(false);
        fireTaskEvent(e);
    }
    
    private void fireTaskEvent(TaskEvent e) {
        if (m_listeners!=null) {
            for (int i=0;i<m_listeners.size();i++) {
                ((TaskListener) m_listeners.get(i)).taskProgressed(e);
            }
        }
        e=null;
    }
    
    private synchronized void addToQueue(Ticket ticket) {
        final int eventId = ticket.getEvent().getID();
        if (m_closing && eventId!=CoffeeEvent.XML_SAVE && eventId!=CoffeeEvent.XML_SAVE_CONFIG && eventId!=CoffeeEvent.SAVE) return;
    	m_queue.add(ticket);
        
    }
    
    public ServerDescriptor getCurrentServer() {
    	NewsServer ns = (NewsServer) getTicketHandler(CoffeeEvent.GET_ARTICLE);
    	ServerDescriptor bsd=null;
    	for (Iterator it = m_servers.keySet().iterator();it.hasNext();) {
    		ServerDescriptor sd = (ServerDescriptor) it.next();
    		if (sd.equals(ns.getServer())) {
    			return sd;
    		} else if (sd.isBinary()) {
    			bsd= sd;
    		}
    	}
    	return bsd;		
    }
    
    public final TicketHandler getTicketHandler(int eventID) {
       return (TicketHandler) m_actions.get(new Integer(eventID));
    }
    
    private final int getSequence(int eventId) {
        for (int i=0;i<SEQUENCE.length;i+=3) {
            if (eventId==SEQUENCE[i]) {
                return SEQUENCE[i+1];
            }
        }
        return -1;
    }
    
    private final boolean isCallBack(int eventId) {
        for (int i=0;i<SEQUENCE.length;i+=3) {
            if (eventId==SEQUENCE[i] && SEQUENCE[i+2]!=0) {
                return true;
            }
        }
        return false;
    }
    
    public void registerHandler(TicketHandler handler) {
        int[] keys = handler.getHandledEvents();
        for (int i=0;i<keys.length;i++) {
            m_actions.put(new Integer(keys[i]), handler);
        }
        try {
            handler.open();
        } catch (Exception ex) {}
    }

    public NewsServer getServer(ServerDescriptor sd) {
    	NewsServer ns = (NewsServer) m_servers.get(sd);
    	if (ns==null) {
    		ns = new NewsServer(sd);
    		m_servers.put(sd,ns);
    	}
    	return ns;
    }
    
	public List getQueList() {
		return m_queue.getList();
	}
    
}
