/* 	Decoder.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 5-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.decode;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.local.FileMessage;

/** 
 * Interface to define a Decoder as obtained
 * by the {@link DecoderFactory} on the basis of the 
 * {@link Article} supplied
 * <p>Due to the difference in implementation of the Decoders
 * the actual decoding is done either when a piece is added or on
 * a call to {@link #getFileMessage()}</p> 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 1.0 25-08-2005
 * @see DecoderFactory
 */
public interface Decoder {
	
	/**
	 * hand a piece of coded characters to the Decoder
	 * 
	 * @param s String, a piece or the complete encoded
	 * characters to decode
	 * @throws Exception, thrown if the Piece cannot be
	 * added for decoding
	 * @see #getFileMessage()
	 */
    public void decodePiece(String s) throws Exception;
    
    /**
     * get the {@link FileMessage} with a reference to 
     * the decoded bytes
     * @return {@link FileMessage} the reference to the decoded
     * bytes and possible file information
     * @throws Exception, thrown if the decoder cannot decode
     * the file or FileMessage cannot be created
     */
    public FileMessage getFileMessage() throws Exception;
}
