/* 	FileConnector.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 23-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		Changes for storing files in different locations, depending on mimwe-type
 * TODO Class should be seriously cleaned up		
*/
package org.brains2b.coffee.file;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Vector;

import org.brains2b.coffee.CoffeeSpill;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.TicketHandler;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.MultiPartGroup;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.monitor.ProgressMonitor;
import org.brains2b.coffee.plugin.MimeType;
import org.brains2b.coffee.plugin.nzb.NzbDocument;
import org.brains2b.coffee.plugin.search.QueryHandler;
import org.brains2b.saxo.SAXReader;
import org.brains2b.saxo.SAXWriter;
import org.xml.sax.InputSource;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class FileConnector implements TicketHandler {
    
    private final static int[] EVENTS = new int[] {
        CoffeeEvent.XML_READ,
        CoffeeEvent.XML_READ_CONFIG,
        CoffeeEvent.XML_SAVE,
        CoffeeEvent.XML_SAVE_CONFIG,
        CoffeeEvent.NEWS_GROUP,
        CoffeeEvent.SAVE
    };
    
    
    private File m_path;
	private Vector m_monitors;
    
    private static HashMap m_handlers=new HashMap();
    
    static {
        m_handlers.put("text/xml.nzb", NzbDocument.class);
        m_handlers.put("text/xml.query", QueryHandler.class);
    }
    /**
     * 
     */
    public FileConnector(String name,File path) {
        super();
		makePath(name,path);
    }
    
    public FileConnector(File path) {
        m_path=new File(path,"X");
    }

    private void makePath(String name, File path) {
        if (!name.equals(path.getName())) {
            m_path=new File(path,name);
        } else {
            m_path=path;
        }
        if (!m_path.exists()) {
            m_path.mkdirs();
        }
    }
    
    public NNTPMessage getFile(Class c, String name) throws Exception {
        File file=new File(m_path,name+".xml");
        FileContentHandler fch=getContentHandler(c);
        //fch.setMessageName(name);
        NNTPMessage result= getFile(fch,file);
        
        return result;
        
    }
    
    public NNTPMessage getRootFile(FileContentHandler fch, String name) throws Exception {
        File file=new File(m_path.getParent(),name+".xml");
        NNTPMessage result= getFile(fch,file);
        
        return result;
        
    }
    
    /*public NNTPMessage getFilteredFile(Class c, String name, MessageFilter filter) throws Exception {
        File file=new File(m_path,name+".xml");
        FileContentHandler fch=getContentHandler(c);
        fch.setMessageName(name);
        fch.setFilter(filter);
        return getFile(fch,file);
        
    }*/
    
    public NNTPMessage getFile(FileContentHandler fch, File file) throws Exception {
    	if (file.exists()) {
        	return getFile(fch,new FileInputStream(file));
    	} else {
    	    return null;
    	}
    }
    public NNTPMessage getFile(FileContentHandler fch, byte[] b ) throws Exception {
        return getFile(fch,new ByteArrayInputStream(b));
    }
      	
   	private NNTPMessage getFile(FileContentHandler fch, InputStream is) throws Exception {
            SAXReader p=new SAXReader();
     	    if (m_monitors!=null) {
     	        for (int i=0;i<m_monitors.size();i++) {
     	            fch.addProgressMonitor((ProgressMonitor) m_monitors.get(i));
     	        }
     	    }
            p.setContentHandler(fch);
     	    p.parse(new InputSource(is));
     	    is.close();
     	    return fch.getMessage(); 
    }
    
    public void save (File path, NNTPMessage message, String name) throws Exception {
        File saving = null;
    	if (message instanceof Article && message.getId()==null) {
        	path = new File(path.getParentFile().getParent(),"outbox");
        	saving = File.createTempFile("post", ".xml",path);
        	((Article) message).setId(saving.getName());
    	} else if (message instanceof Article) {
        	path = new File(path.getParentFile().getParent(),"saved");
        	saving = File.createTempFile("post", ".xml",path);
        	((Article) message).setId(saving.getName());
    	} else if (message instanceof MimeType) {
    		saving = new File(path.getParent(),getContentHandler(((MimeType) message).getMimeType()).getDefaultDir());
  			saving = new File(saving, name +".xml");	  		
    	} else {
        	saving=new File(path,name+".xml");
        }
        FileWriter fw=new FileWriter(saving);
        SAXWriter sw=new SAXWriter(fw);
        sw.startDocument();
        message.writeMessage(sw);
        sw.endDocument();
    }
        
    public void save(FileMessage fm, File path) throws IOException {
        
        if (fm.getTempFile()!=null) {
            /*FileInputStream fis=new FileInputStream(fm.getTempFile());
            int b=-1;
            while ((b=fis.read())!=-1) {
               fos.write(b); 
            }
            fis.close();
            fos.close();*/
            
            // Create channel on the source
            FileChannel srcChannel = new FileInputStream(fm.getTempFile()).getChannel();
        
            // Create channel on the destination
            FileChannel dstChannel = new FileOutputStream(new File(path,fm.getFile().getPath())).getChannel();
        
            // Copy file contents from source to destination
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
        
            // Close the channels
            srcChannel.close();
            dstChannel.close();
            fm.getTempFile().delete();
           
        } else {
            FileOutputStream fos=new FileOutputStream(new File(path,fm.getFile().getPath()));
            fos.write(fm.getData());
            fos.close();
            fm.setData(null);
        }
        
    }
    
    private FileContentHandler getContentHandler(String mime) throws CoffeeSpill {
        try {
            Class c= (Class) m_handlers.get(mime);
            if (c!=null) {
                FileContentHandler fh= (FileContentHandler) c.newInstance();
                return fh;
            }
        } catch (Exception ex) {
           throw new CoffeeSpill("Cannot find a way to load file for mime-type: " + mime,ex); 
        }
        throw new CoffeeSpill("Cannot find loader for mime-type: " + mime);
    }
    
    private FileContentHandler getContentHandler(Class c) {
        if (c==NewsGroupImpl.class || c==MultiPartGroup.class) {
        	return new NewsGroupHandler();
        } else if (c==GroupList.class) {
        	return new GroupListHandler();
        } else if (c==FilterList.class) {
            return new FilterHandler(FilterList.getInstance());
        } else {
            return null;
        }
    }
    
    public void addProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors==null) {
            m_monitors=new Vector();
        }
        m_monitors.add(monitor);
    }
    
    public void removeProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors!=null) {
            m_monitors.remove(monitor);
        }
    }

    public File getPath() {
        return m_path;
    }

    public void open() throws Exception {
        
    }
    public void close() throws Exception {
        
    }

    public NNTPMessage handleEvent(CoffeeEvent event) throws CoffeeSpill {
        NNTPMessage message =null;
        try {
        switch (event.getID()) {
            case CoffeeEvent.XML_READ_CONFIG:
                CoffeeMenu c = (CoffeeMenu) event.getArgument();
                CoffeeMenuHandler cmh=new CoffeeMenuHandler();
                cmh.setList(c);
                message=getRootFile(cmh,CoffeeMenu.makeMultiple(c.getId()));
                break;
            case CoffeeEvent.XML_READ:
                if (event.getArgument() instanceof ServerDescriptor) {
                    message = getFile(GroupList.class,"groups");
                    if (message==null) {
                        fireCoffeeEvent(event.getExchange(),CoffeeEvent.GET_GROUPLIST,event.getArgument());
                    }
                } else if (event.getArgument() instanceof FilterList) {
                	FileContentHandler fh= getContentHandler(FilterList.class);
                	return getFile(fh, new File(m_path.getParentFile(),"filters.xml"));
                	
                } else {
                    FileMessage fm= (FileMessage) event.getArgument();
                    FileContentHandler fh= getContentHandler(fm.getMimeType());
                    if (fm.getData()!=null) {
                        message=getFile(fh,fm.getData());
                    } else if (fm.getTempFile()!=null){
                        message=getFile(fh,
                                fm.getTempFile()); 
                    } else {
                        message=getFile(fh,
                                fm.getFile()); 
                    }
                    fm.setContent(fh);
                    message=fm;
                }
                break;
            case CoffeeEvent.NEWS_GROUP:
                String groupName = (String) event.getArgument();
                NewsGroupImpl group= (NewsGroupImpl) getFile(NewsGroupImpl.class,groupName);
                if (group==null) {
                    group = new NewsGroupImpl();
                    group.setName(groupName);
                }
                message=group;
                break;
            case CoffeeEvent.XML_SAVE:
                NNTPMessage msg= (NNTPMessage) event.getArgument();
                if (msg instanceof FilterList) {
                	save(m_path.getParentFile(),msg,msg.getId());
                } else if (!(msg instanceof InfoMessage)){
                	save(m_path,msg,msg.getId());
                }
                break;
            case CoffeeEvent.XML_SAVE_CONFIG:
                msg= (NNTPMessage) event.getArgument();
                save(m_path.getParentFile(),msg,CoffeeMenu.makeMultiple(msg.getId()));
                break;
            case CoffeeEvent.SAVE:
                msg= (NNTPMessage) event.getArgument();
                if (msg instanceof Article) {
                    save(((Article)msg).getFile(),new File(m_path.getParentFile().getParentFile(),"download"));
                    ((Article) msg).setBody(null);
                } else if (msg instanceof FileMessage) {
                	save((FileMessage) msg,new File(m_path.getParentFile().getParentFile(),"download"));
                }
                break;
                
        }
        } catch (CoffeeSpill cs) {
            throw cs;
        } catch (Exception ex) {
            throw new CoffeeSpill("Problem with IO",ex);
        }
        return message;
    }

    public boolean isAvailable() {
       return true;
    }
    
    public int[] getHandledEvents() {
        return EVENTS;
    }
    
    private void fireCoffeeEvent(CoffeeExchange source, int id, Object argument) {
        CoffeeEvent ce= new CoffeeEvent(source,id,argument);
        TicketBroker.getBroker().broker(ce);
    }
}
