/* 	CfFilterDelegate.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 7-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.GroupList;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.FilterGroupList;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.filter.FilterNewsGroup;
import org.brains2b.coffee.message.filter.FilterTarget;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.dialog.DialogHelper;
import org.brains2b.swingext.gui.IconFactory;
import org.brains2b.util.StringHelper;

public class CfFilterDelegate implements ActionListener {
    
    private JToggleButton m_button;
    private JTextField m_filterField;
    private CoffeeExchangeView m_view;
    private NNTPMessage m_message;
    
    private List m_localFilter;
    
    public CfFilterDelegate(CoffeeExchangeView view) {
        m_view=view;
    }
    
    public JPanel addFilter(int eventId,String key,int fieldWidth)  {
        JPanel filter=new JPanel();
        ExtActionMap map=new ExtActionMap(this);
        map.addAction("Filter.find",null,"Filter this list",null,null,'f');
        map.addAction("Filter.findOnKey",null,"Filter this list",null,null,'f');
        map.addAction("Filter.exclude",null,"Exclude from list","/org/brains2b/coffee/resource/FilterExclude16.gif",null,'x');
        map.addAction("Filter.new",null,"Only show new messages","/org/brains2b/coffee/resource/New16.gif",null,'x');
        filter.setActionMap(map);
        filter.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,0),"Filter.findOnKey");
        m_filterField=new JTextField();
        m_filterField.setColumns(fieldWidth);
        m_filterField.setName(key);
        m_button=new JToggleButton(map.get("Filter.find"));
        m_button.setIcon(IconFactory.createIcon("/org/brains2b/coffee/resource/Filter16.gif"));
        m_button.setPreferredSize(new Dimension(24,24));
        
        filter.add(m_filterField);
        filter.add(m_button);
        if (eventId==8017) {
        	JButton exclude=new JButton(map.get("Filter.exclude"));
            exclude.setPreferredSize(new Dimension(24,24));
        	filter.add(exclude);
            filter.add(new JSeparator());
            JToggleButton filterNew = new JToggleButton(map.get("Filter.new"));
            filterNew.setPreferredSize(new Dimension(24,24));
            filter.add(filterNew);
            
        }
        return filter;
    }

    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
    	if (actionCommand.equals("Filter.find")) {
        	if (m_filterField.getText()==null || m_filterField.getText().length()==0) {
                m_button.setSelected(false);
            } else if (!m_button.isSelected()) {
            	m_filterField.setText("");
            }
            MessageFilter filter=new MessageFilter(m_filterField.getName(),m_filterField.getText());
            FilterTarget ft=getFilterTarget(m_message,filter,!m_button.isSelected());
            if (ft!=null) {
                m_view.setCurrentMessage(ft);
            }
        } else if (actionCommand.equals("Filter.findOnKey")) {
            boolean b=!StringHelper.isEmpty(m_filterField.getText());
            m_button.setSelected(b);
            actionPerformed(new ActionEvent(this,ActionEvent.ACTION_PERFORMED,"Filter.find"));
        } else if (actionCommand.equals("Filter.exclude")) {
            showExclusion();
        } else if (actionCommand.equals("Filter.new")) {
    		MessageFilter filter = new MessageFilter("new", "true");
        	FilterTarget ft=getFilterTarget(m_message,filter,!((JToggleButton) e.getSource()).isSelected());
            if (ft!=null) {
                m_view.setCurrentMessage(ft);
            }
        } else if (actionCommand.equals("Filter.poster") || actionCommand.equals("Filter.posterFromAll")) {
        	String group = "Filter.poster".equals(actionCommand)?m_message.getId():"*";
        	List selection = (List) e.getSource();
        	for (Iterator it = selection.iterator();it.hasNext();) {
        		Article a = (Article) it.next();
        		MessageFilter mf = new MessageFilter("From",true,"equals", a.getHeader().getAuthor(),false);
        		FilterList.getInstance().addFilterFor(group,mf);
        	}
        	FilterTarget ft=getFilterTarget(m_message,null,false);
            if (ft!=null) {
                m_view.setCurrentMessage(ft);
            }
        }
        
    }

    public void setCurrentMessage(NNTPMessage message) {
        if (!(message instanceof FilterTarget)) {
            m_message = message;
        }
    }
    
    public FilterTarget getFilterTarget(NNTPMessage message, MessageFilter filter, boolean remove) {
        if (m_localFilter==null) {
        	m_localFilter = new ArrayList();
        }
        if (filter!=null) {
        	for (Iterator it = m_localFilter.iterator();it.hasNext();) {
        		MessageFilter mf = (MessageFilter) it.next();
        		if (mf.getKey().equals(filter.getKey())) {
        			it.remove();
        		}
        	}
        	if (!remove)
        	m_localFilter.add(filter);
        
        }
    	if (message instanceof GroupList) {
            return new FilterGroupList((GroupList) message,m_localFilter);
        } else if (message instanceof NewsGroup) {
            return new FilterNewsGroup((NewsGroup) message,m_localFilter);
        } else {
            return null;
        }
    }
    
    public void clearFilter() {
         m_filterField.setText("");
         m_button.setSelected(false);
    }
    
    private void showExclusion() {
        CfFilterPane pane=new CfFilterPane(m_view);
        pane.setFilters(FilterList.getInstance());
        pane.setFilterContext(m_message);
        DialogHelper.showDialog((JComponent) m_view, pane, "Exclude filters"); 
        
    }
    
}
