/* 	CfFilterRow.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 26, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;
import org.brains2b.swingext.gui.SelectablePanel;
import org.brains2b.util.StringHelper;

/**
 * GUI Panel that acts both as a Pane for a dialog and as row
 * in {@link CfFilterPane} to edit {@link MessageFilter} statements
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfFilterRow extends JPanel implements SelectablePanel, Form {
    
	private JComboBox m_field;
    private JComboBox m_actions;
    private JTextField m_value;
    private JCheckBox m_kill;
    private boolean m_selected;
    
    public CfFilterRow() {
        this(false);
    }
    
    public CfFilterRow(boolean asDialog) {
        super();
        init(asDialog);
    }
    
     public CfFilterRow(MessageFilter mf) {
        this (mf,false);
    }
     
    public CfFilterRow(MessageFilter mf, boolean asDialog) {
        super();
		init(asDialog);
        setMessageFilter(mf);
    }
    
    private void init(boolean asDialog) {
    	JPanel main = this;
    	if (asDialog) {
    		main = new JPanel();
    	}
        m_field=new JComboBox(ArticleHeader.TYPES);
        main.add(m_field);
        m_actions=new JComboBox(MessageFilter.OPERATIONS);
        main.add(m_actions);
        m_value=new JTextField();
        m_value.setColumns(20);
        main.add(m_value);
        if (!asDialog) {
	        m_kill=new JCheckBox("kill");
	        main.add(m_kill);
        }
        if (asDialog) {
        	setLayout(new BorderLayout());
        	add(main,BorderLayout.CENTER);
        	add(new FormAction(this,FormAction.BTN_OK_CANCEL),BorderLayout.SOUTH);
        } else {
        	Dimension d=new Dimension(512,40);
        	setPreferredSize(d);
        	setMinimumSize(d);
        	setMaximumSize(new Dimension(1024,40));
        	setBorder(BorderFactory.createEtchedBorder());
        }
    }
    
    public void setMessageFilter(MessageFilter mf) {
        m_field.setSelectedItem(mf.getKey());
        m_actions.setSelectedIndex(mf.getOperator());
        m_value.setText(mf.getValue()!=null?mf.getValue().toString():null);
        m_kill.setSelected(mf.isKill());
    }
    
    public MessageFilter getMessageFilter() {
        if (StringHelper.isEmpty(m_value.getText())) {
        	return null;
        }
        MessageFilter mf = new MessageFilter();
        mf.setKey((String) m_field.getSelectedItem());
        mf.setOperator(m_actions.getSelectedIndex());
        mf.setValue(m_value.getText());
        if (m_kill!=null)
        mf.setKill(m_kill.isSelected());
        mf.setNot(true);
        return mf;
    }

	public boolean isSelected() {
		return m_selected;
	}

	public void setSelected(boolean b) {
		m_selected = b;			
	}

	public void performApply() {
		// Not implemented
	}

	public void performCancel() {
		m_value.setText(null); //This will return MessageFilter as null;
		SwingUtilities.getWindowAncestor(this).dispose();
		
	}

	public void performOk() {
		SwingUtilities.getWindowAncestor(this).dispose();
		
	}

	public void performUndo() {
		// Not implemented
		
	}
    
}