/* 	CfFrame.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		call de/activate on setVisible(true), implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfFrame extends JFrame implements CoffeeExchangeView, TaskListener {
    
	class TabHandler implements ChangeListener {

		private CoffeeExchangeView m_last;
		
		public void stateChanged(ChangeEvent e) {
			
			CoffeeExchangeView view = (CoffeeExchangeView) m_tabPane.getSelectedComponent();
			if (m_last!=null) {
				m_last.deactivate();
			} 
			m_serverPane.setSelectEvent(224,view.getSelectEvent(224));
			m_serverPane.setSelectEvent(Subscription.SUBSCRIPTION,view.getSelectEvent(Subscription.SUBSCRIPTION));
			view.activate();
			m_last = view;
			
		}
		
	}
	
    CfStatusBar m_bar=new CfStatusBar();
    ResourceBundle m_resource;
	JTabbedPane m_tabPane=null;
	JSplitPane m_split=null;
	ExtActionMap m_actionMap;
	CfServerPane m_serverPane;
	CfMessagePane m_messagePane;
    /**
     * @throws java.awt.HeadlessException
     */
    public CfFrame(ExtActionMap map) {
        super();
        setActionMap(map);
        init();
    }

    public ExtActionMap getActionMap() {
        return m_actionMap;
    }
    public void setActionMap(ExtActionMap actionMap) {
        m_actionMap = actionMap;
    }

    private void init() {
        m_resource = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
        m_split=new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        m_serverPane=new CfServerPane(this);
        m_split.setLeftComponent(m_serverPane);
        m_tabPane=new JTabbedPane();
        m_messagePane=new CfMessagePane();
        m_tabPane.addTab("Message",m_messagePane);
        m_tabPane.addChangeListener(new TabHandler());
        m_split.setRightComponent(m_tabPane);
        getContentPane().add(m_bar,BorderLayout.SOUTH);
        getContentPane().add(m_split,BorderLayout.CENTER);
        JMenuBar menu=new JMenuBar();
        
        JMenu[] menus=getMenus();
        for (int i=0;i<menus.length;i++) {
            menu.add(menus[i]);
        }
        menus=m_serverPane.getMenus();
        for (int i=0;i<menus.length;i++) {
        	menu.add(menus[i]);
        }
        
        menus=m_messagePane.getMenus();
        for (int i=0;i<menus.length;i++) {
            menu.add(menus[i]);
        }
        setJMenuBar(menu);
        
    }
    
    /* (non-Javadoc)
     * @see java.awt.Window#show()
     */
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (b) {
        	activate();
        } else {
        	deactivate();
        }
    }
    
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

    }
    public void setCurrentMessage(NNTPMessage message) {
		m_serverPane.setCurrentMessage(message);
		CoffeeExchangeView view= (CoffeeExchangeView) m_tabPane.getSelectedComponent();
		if (view!=null) {
		    view.setCurrentMessage(message);
		}

    }
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#getMenus()
     */
    public JMenu[] getMenus() {
        JMenu file=new JMenu(m_resource.getString("Menu.File"));
        file.add(getActionMap().get("System.Settings"));
        
        file.add(getActionMap().get("System.Exit"));
        
        return new JMenu[] {file};
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#activate()
     */
    public void activate() {
		 m_split.setDividerLocation(0.25d);
         m_serverPane.activate();
		 CoffeeExchangeView view = (CoffeeExchangeView) m_tabPane.getSelectedComponent();
         view.activate();
		 m_serverPane.setSelectEvent(224,view.getSelectEvent(224));
		 m_serverPane.setSelectEvent(Subscription.SUBSCRIPTION,view.getSelectEvent(Subscription.SUBSCRIPTION));
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.plugin.CoffeeExchangeView#deactivate()
     */
    public void deactivate() {
		for (int i=0;i<m_tabPane.getTabCount();i++) {
            CoffeeExchangeView view = (CoffeeExchangeView) m_tabPane.getComponentAt(i);
            view.deactivate();
        }
		m_serverPane.deactivate();
    }
    
    public void showPlugin(String title) {
    	for (int i=0;i<m_tabPane.getTabCount();i++) {
            if (title.equals(m_tabPane.getTitleAt(i))) {
                m_tabPane.setSelectedIndex(i);
                break;
            }
        }
    }
    
    public int registerPlugin(String title,CoffeeExchangeView view) {
        m_tabPane.addTab(title,(JPanel) view);
        return m_tabPane.getTabCount();
    }

	public void taskProgressed(TaskEvent e) {
		m_bar.taskProgressed(e);
		
	}

	public int getSelectEvent(int messageType) {
		return CoffeeEvent.NONE;
	}
    
}
