/* 	CfHeaderPane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 12-aug-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui;

import java.awt.Color;
import java.awt.FontMetrics;

import javax.swing.BorderFactory;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.log.Logger;
import org.brains2b.swingext.gui.IconFactory;

public class CfHeaderPane extends JTextPane {

    public CfHeaderPane() {
        super();
        init();
    }
    
    private void init() {
        setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createLineBorder(new Color(0xb0,0xb0,0xb0)),
                BorderFactory.createEmptyBorder(5,5,5,5)));
        registerHeaderStyles(getStyledDocument());
    }
    
    
    private void registerHeaderStyles(StyledDocument doc) {
        
        Style def = StyleContext.getDefaultStyleContext().
                        getStyle(StyleContext.DEFAULT_STYLE);
        
        Style regular = doc.addStyle("regular", def);
        Style file= doc.addStyle("photo",def);
        StyleConstants.setIcon(file,IconFactory.createIcon("/org/brains2b/coffee/resource/Text24.gif"));
        
        Style heading= doc.addStyle("heading",regular);
        StyleConstants.setForeground(heading,new Color(0xb0,0xb0,0xb0));
        StyleConstants.setBold(heading,true);
        
        doc.addStyle("value",regular);
    }

    public void setHeader(Article a) {
         int pos=0;
         ArticleHeader header=a.getHeader();
         StyledDocument doc=getStyledDocument();
         
         try {
            doc.remove(0,doc.getLength());
            doc.insertString(pos,"Subject:\t",doc.getStyle("heading"));
            doc.insertString(doc.getLength(),header.getSubject()+"\n",doc.getStyle("value"));
            doc.insertString(doc.getLength(),"From:\t",doc.getStyle("heading"));
            doc.insertString(doc.getLength(),header.getAuthor()+"\n",doc.getStyle("value"));
            doc.insertString(doc.getLength(),"Date:\t",doc.getStyle("heading"));
            doc.insertString(doc.getLength(),header.getDate()+"\n",doc.getStyle("value"));
            doc.insertString(doc.getLength(),"Newsgroups:\t",doc.getStyle("heading"));
            formatNewsGroups(doc,header.getHeaderValue("Newsgroups"));
            
            FontMetrics fm=getFontMetrics(getFont());
            int tabsize=fm.charWidth('w')*10;
            TabStop[] tabs=new TabStop[10];
            for (int i=0;i<tabs.length;i++) {
                tabs[i]=new TabStop(i*tabsize);
            }
            SimpleAttributeSet atts=new SimpleAttributeSet();
            StyleConstants.setTabSet(atts,new TabSet(tabs));
            doc.setParagraphAttributes(0,doc.getLength(),atts,true);
            
         } catch (BadLocationException blex) {
             Logger.printException(blex);
         }
         
     }

    private void formatNewsGroups(StyledDocument doc, String newsGroups) throws BadLocationException {
        newsGroups=newsGroups.replaceAll(",",",\t");
        doc.insertString(doc.getLength(),newsGroups,doc.getStyle("value"));
        
    }
    

}
