/* 	CfMessagePane.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		Moved all JTable code to ArticleTable class,
 * 										implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.table.ArticleNode;
import org.brains2b.coffee.gui.table.ArticleTable;
import org.brains2b.coffee.gui.table.TreeMaker;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.NewsGroupImpl;
import org.brains2b.coffee.message.filter.FilterList;
import org.brains2b.coffee.message.filter.FilterNewsGroup;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.swingext.dialog.DialogHelper;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class CfMessagePane extends JPanel implements CoffeeExchangeView,ActionListener {
	
	class SelectionHandler implements ListSelectionListener {
        
        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                ListSelectionModel model = (ListSelectionModel) e.getSource();
                if (model.getMinSelectionIndex()==model.getMaxSelectionIndex()) {
                    Article a=m_table.getArticleModel().getArticle(model.getMinSelectionIndex());
                    if (a!=null) {
                        a.setNewsGroup(m_newsGroup.getName());
                        handleGetArticle(a);
                        getActionMap().get("Message.followup").setEnabled(true);
                    } else {
                        getActionMap().get("Message.followup").setEnabled(false);
                    }
                }
            }

        }
    }
	
    ArticleTable m_table;
    private CfPreviewPane m_preview;
    private JSplitPane m_split;
   	private NewsGroup m_newsGroup;
   	private CfFilterDelegate m_filter;
    private CfStatsDelegate m_stats;
    public CfMessagePane() {
        super();
        init();
    }
    
    private void init() {
       	setLayout(new BorderLayout());
        m_filter=new CfFilterDelegate(this);
        m_stats=new CfStatsDelegate(this);
        m_split=new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    	
    	ExtActionMap map=new ExtActionMap(this);
    	map.addAction("save.attach","Save attachements",null,null,"ctrl shift s",'s');
    	map.addAction("save.message","Save message",null,null,"ctrl s",'s');
        map.addAction("newsgroup.refresh","Refresh",null,"/org/brains2b/coffee/resource/Refresh16.gif","ctrl r",'r');
        map.addAction("Message.new","Post message",null,null,"ctrl n",'n');
        map.addAction("Message.followup","Post follow-up",null,null,"ctrl f",'f');
       	map.addAction("Filter.poster","Exclude poster",null,null,"ctrl k",'0');
       	map.addAction("Filter.posterFromAll","Exclude posterFromAll",null,null,"ctrl shift k",'0');
       	map.addAction("Filter.message","Exclude ...",null,null,"ctrl shift e",'0');
        
       	map.get("Message.followup").setEnabled(false);
        setActionMap(map);
    	
        m_table = new ArticleTable();
        m_table.setActionMap(map);
        m_table.getSelectionModel().addListSelectionListener(new SelectionHandler());
        JPanel listPane=new JPanel(new BorderLayout());
        JScrollPane tablePane=new JScrollPane(m_table);
        listPane.add(tablePane,BorderLayout.CENTER);
        listPane.add(m_stats.addStats(NewsGroupImpl.class),BorderLayout.SOUTH);
       	m_preview=new CfPreviewPane(this);
       	JScrollPane previewPane=new JScrollPane();
       	previewPane.setViewportView(m_preview);
       	
       	m_split.setTopComponent(listPane);
       	m_split.setBottomComponent(previewPane);
        JPanel top = new JPanel();
        add(top,BorderLayout.NORTH);
        map.get("newsgroup.refresh").setEnabled(false);
        JButton btn= new JButton(map.get("newsgroup.refresh"));
        btn.setText(null);
        btn.setPreferredSize(new Dimension(24,24));
        top.add(btn);
        top.add(m_filter.addFilter(8017,"Subject",30));
       	add(m_split,BorderLayout.CENTER);

    }
   
    public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
        TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));
    }
    public void setCurrentMessage(NNTPMessage message) {
        if (message instanceof NewsGroup) {
            m_newsGroup=(NewsGroup) message;
            FilterNewsGroup fng = new FilterNewsGroup(m_newsGroup,FilterList.getInstance().getAllFilterFor(m_newsGroup.getId()));
            ArticleNode node=TreeMaker.order(fng.getArticleList());
            m_table.getArticleModel().setData(node);
            m_filter.setCurrentMessage(message);
            m_stats.setCurrentMessage(message);
            getActionMap().get("newsgroup.refresh").setEnabled(true);
        } else if (message instanceof Article) {
        	if (m_table.getSelectedRowCount()==1) {
            	Article a=(Article) message;
            	Article al = (Article) m_table.getArticleModel().getArticle(m_table.getSelectedRow());
                if (a.getId().equals(al.getId())) {
                   al.setNew(false);
            	   m_preview.setArticle(a);
                }
            }
        } else if (message instanceof InfoMessage && message.getType()==423) {
        	
        }
    }
    private void handleGetArticle(Article a) {
        a.setNewsGroup(m_newsGroup.getName());
        fireCoffeeEvent(this,CoffeeEvent.GET_ARTICLE,a);
    }
    
    public JMenu[] getMenus() {
        JMenu menu[]= new JMenu[1];
        menu[0]= new JMenu("Message");
        menu[0].add(getActionMap().get("Message.new"));
        menu[0].add(getActionMap().get("Message.followup"));
        return menu;
    }
    
    public void activate() {
        m_split.setDividerLocation(0.40d);
    }
    
    public void deactivate() {
        
    }
    
    public void actionPerformed(ActionEvent e) {
    	String actionCommand = e.getActionCommand();
    	if (actionCommand.equals("save.attach")) {
    		int[] rows = m_table.getSelectedRows();
            for (int i=0;i<rows.length;i++) {
              	fireCoffeeEvent(this,CoffeeEvent.SAVE_ATTACHMENT,m_table.getArticleModel().getArticle(rows[i]));
            }	
    	} else if (actionCommand.equals("save.message")) {
    		int[] rows = m_table.getSelectedRows();
            for (int i=0;i<rows.length;i++) {
              	fireCoffeeEvent(this,CoffeeEvent.SAVE_POST,m_table.getArticleModel().getArticle(rows[i]));
            }
    	} else if (actionCommand.startsWith("Filter")) {
    		int[] rows = m_table.getSelectedRows();
            List selected = new ArrayList(rows.length);
    		for (int i=0;i<rows.length;i++) {
              	selected.add(m_table.getArticleModel().getArticle(rows[i]));
            }
    		m_filter.actionPerformed(new ActionEvent(selected,e.getID(),actionCommand));
    	} else if (actionCommand.equals("newsgroup.refresh")) {
            fireCoffeeEvent(this, CoffeeEvent.GET_ARTICLES, m_newsGroup);
        } else if (actionCommand.equals("Message.new")) {
            CfEditMessage msg=new CfEditMessage();
            msg.setCurrentMessage(m_newsGroup);
            DialogHelper.showDialog(this, msg, "New Message");
            
        } else if (actionCommand.equals("Message.followup")) {
            CfEditMessage msg=new CfEditMessage();
            msg.setCurrentMessage(m_preview.getArticle());
            DialogHelper.showDialog(this, msg, "Post followup message");
        }

    }

	public int getSelectEvent(int messageId) {
		switch (messageId) {
		case 224:
		case Subscription.SUBSCRIPTION:
			return CoffeeEvent.NEWS_GROUP;
		default:
			return CoffeeEvent.NONE;
		}
	}
    
}
