/* 	CfQueView.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sep 4, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import org.brains2b.coffee.Ticket;
import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.action.ExtActionMap;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;

public class CfQueView extends JPanel implements CoffeeExchangeView, TaskListener, ActionListener {

	class SelectionHandler implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent e) {
			if (!e.getValueIsAdjusting()) {
				Object[]  keys = getActionMap().keys();
				for (int i=0;i<keys.length;i++) {
					if (!((String) keys[i]).endsWith("All")) {
						getActionMap().get(keys[i]).setEnabled(true);
					}
				}
			}
		}
		
	}
	
	class QueTableModel extends AbstractTableModel {

		public int getColumnCount() {
			return 3;			
		}

		public int getRowCount() {
			return m_queue==null?0:m_queue.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {
			Ticket t = (Ticket) m_queue.get(rowIndex);
			CoffeeEvent event = t.getEvent();
			switch (columnIndex) {
				case 0:
					return getAction(t);
				case 1:
					return event.getArgument().toString();
				case 2:
					return String.valueOf(t.getSequence());
					
			}
			return null;
		}
		
	}
	
	private JTable m_table;
	private JPopupMenu m_popup;
	private List m_queue;
	
	public CfQueView() {
		super();
		init();
	}
	
	private void init() {
		setPreferredSize(new Dimension(600,400));
		setLayout(new BorderLayout());
		ExtActionMap map = new ExtActionMap(this);
		setActionMap(map);
		map.addAction("Queue.up", "up", "Move the action higher in the queue", "/org/brains2b/coffee/resource/Up16.gif", null, 'p');
		map.addAction("Queue.down", "down", "Move the action lower in the queue", "/org/brains2b/coffee/resource/Down16.gif", null, 'p');
		map.addAction("Queue.pause", "pause", "pause the action in the queue", "/org/brains2b/coffee/resource/Pause16.gif", null, 'p');
		map.addAction("Queue.pauseAll", "pause all", "pause all actions in the queue", "/org/brains2b/coffee/resource/PauseAll16.gif", null, 'p');
		map.addAction("Queue.resume", "resume", "resume the paused action in the queue", "/org/brains2b/coffee/resource/Play16.gif", null, 'p');
		map.addAction("Queue.resumeAll", "resume all", "resume all paused actions in the queue", "/org/brains2b/coffee/resource/PlayAll16.gif", null, 'p');
		map.addAction("Queue.remove", "remove", "remove this action from the queue", "/org/brains2b/coffee/resource/Remove16.gif", "Del", 'r');
		map.addAction("Queue.removeAll", "remove all", "remove  all the actions in the queue", "/org/brains2b/coffee/resource/RemoveAll16.gif", "Control Del", 'r');
		
		JToolBar bar = new JToolBar(JToolBar.HORIZONTAL);
		m_popup = new JPopupMenu();
		Object[] keys= map.allKeys();
		for (int i=0;i<keys.length;i++) {
			Action action = getActionMap().get(keys[i]);
			JButton btn = new JButton(action);
			btn.setText(null);
			bar.add(btn);
			if (!((String)keys[i]).endsWith("All")) {
				action.setEnabled(false);
				m_popup.add(new JMenuItem(action));
			}
		}
		add(bar,BorderLayout.NORTH);
		m_table = new JTable();
		m_table.setModel(new QueTableModel());
		m_table.getSelectionModel().addListSelectionListener(new SelectionHandler());
		DefaultTableColumnModel colModel = new DefaultTableColumnModel();
		TableColumn tc = new TableColumn(0,30);
		tc.setHeaderValue("Action");
		colModel.addColumn(tc);
		tc = new TableColumn(1,240);
		tc.setHeaderValue("Description");
		colModel.addColumn(tc);
		tc = new TableColumn(2,30);
		tc.setHeaderValue("Seq#");
		colModel.addColumn(tc);
		m_table.setColumnModel(colModel);
		add(new JScrollPane(m_table),BorderLayout.CENTER);
	}
	
	public void setQueue(List queue) {
		m_queue = queue;	
	}
	
	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();
		if ("Queue.remove".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_REMOVE,getSelectedTickets());
		} else if ("Queue.removeAll".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_REMOVE,m_queue);
		} else if ("Queue.up".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_UP,getSelectedTickets());
		} else if ("Queue.down".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_DOWN,getSelectedTickets());
		} else if ("Queue.pause".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_PAUSE,getSelectedTickets());
		} else if ("Queue.pauseAll".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_PAUSE,m_queue);
		} else if ("Queue.resume".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_RESUME,getSelectedTickets());
		} else if ("Queue.resumeAll".equals(action)) {
			fireCoffeeEvent(this,CoffeeEvent.QUEUE_RESUME,m_queue);
		} 
		
	}
	
	public void taskProgressed(TaskEvent e) {

	}

	public void activate() {
		
	}

	public void deactivate() {
		
	}

	public void fireCoffeeEvent(CoffeeExchange source, int action, Object argument) {
		if (argument==null) {
			return;
		}
		TicketBroker broker = TicketBroker.getBroker();
		broker.broker(new CoffeeEvent(source,action,argument));
	}

	public JMenu[] getMenus() {
		return null;
	}

	public void setCurrentMessage(NNTPMessage message) {
		((QueTableModel) m_table.getModel()).fireTableDataChanged();
	}
	
	private Ticket[] getSelectedTickets() {
		int size =m_table.getSelectedRowCount();
		if (size==0) return null;
		Ticket[] tickets = new Ticket[size];
		for (int i=0;i<size;i++) {
			tickets[i]=(Ticket) m_queue.get(m_table.getSelectedRows()[i]);
		}
		return tickets;
	}
	
	private String getAction(Ticket t) {
		if (t.isFinished()) {
			switch (t.getResult()) {
			case Ticket.RESULT_OK: return "Done";
			case Ticket.RESULT_ERROR: return "Error";
			case Ticket.RESULT_REMOVE: return "Killed";
			default: return "Warning";
			}
		} else if (t.isWait()) {
			return "Waiting";
		} return "";
	}

	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
