/* 	CfBinarySend.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sep 4, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		implemented CoffeeExchangeView.getSelectEvent()		
*/
package org.brains2b.coffee.gui.binary;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.event.CoffeeExchange;
import org.brains2b.coffee.gui.CfStatsDelegate;
import org.brains2b.coffee.gui.list.ExtMutableList;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.local.CoffeeBean;
import org.brains2b.coffee.message.local.FileListMessage;
import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.coffee.message.local.Options;
import org.brains2b.coffee.plugin.CoffeeExchangeView;
import org.brains2b.swingext.form.Form;
import org.brains2b.swingext.form.FormAction;

public class CfBinarySend extends JPanel implements CoffeeExchangeView, ActionListener, Form, ChangeListener {

	class SelectionHandler implements ListSelectionListener {

		public void valueChanged(ListSelectionEvent e) {
			if (!e.getValueIsAdjusting()) {
				FileMessage fm = new FileMessage();
				File f= (File) ((JList) e.getSource()).getSelectedValue();
				fm.setFile(f);
				m_binOptions.setCurrentMessage(fm);
			}
			
		}
		
		
	}
	private ExtMutableList m_fileList;
	private BinaryOptions m_binOptions;
	private CfStatsDelegate m_stats;
	private Article m_article;
	
	public CfBinarySend() {
		super();
		init();
	}
	
	public void activate() {
		m_binOptions.activate();
	}

	public void deactivate() {
		// 	Don't call deactivate on m_binOptions while we
		//  are not saving the option changes

	}

	public void fireCoffeeEvent(CoffeeExchange source, int action,
			Object argument) {
		TicketBroker.getBroker().broker(new CoffeeEvent(source,action,argument));

	}

	public JMenu[] getMenus() {
		return null;
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof Article) {
			m_article = (Article) message;
			if (m_article.getFile()!=null) {
				if (m_article.getFile() instanceof FileListMessage) {
					m_fileList.setData(((FileListMessage)m_article.getFile()).getFiles());
				} else {
					m_fileList.addItem(m_article.getFile().getFile());
				}
			}
			m_stats.setCurrentMessage(m_article);
			m_binOptions.setCurrentMessage(m_article);
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("ExtMutableList.Add")) {
			JFileChooser jfc = new JFileChooser();
			jfc.setMultiSelectionEnabled(true);
			int result = jfc.showDialog(this, "Attach");
			if (result==JFileChooser.APPROVE_OPTION) {
				File[] f = jfc.getSelectedFiles();
				for (int i=0;i<f.length;i++) {
					m_fileList.addItem(f[i]);
				}
			}
		} else if (e.getActionCommand().equals("ExtMutableList.Remove")) {
			m_fileList.removeItem(m_fileList.getSelectedIndex());
 		}
		
	}
	
	private void init() {
		setLayout(new BorderLayout());
		m_fileList = new ExtMutableList(ExtMutableList.ACTION_ADD+ExtMutableList.ACTION_REMOVE);
		m_fileList.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createEmptyBorder(5, 5, 5, 5), "Files:"));
		m_fileList.setVisibleRowCount(15);
		m_fileList.setFixedCellWidth(150);
		m_fileList.addActionListener(this);
		m_fileList.addListSelectionListener(new SelectionHandler());
		add(m_fileList,BorderLayout.CENTER);
		m_binOptions = new BinaryOptions();
		m_binOptions.addChangeListener(this);
		m_binOptions.showFormButtons(false);
		JPanel east = new JPanel(new BorderLayout());
		east.add(m_binOptions,BorderLayout.CENTER);
		m_stats = new CfStatsDelegate(this);
		east.add(m_stats.addStats(Article.class),BorderLayout.SOUTH);
		add(east,BorderLayout.EAST);
		add(new FormAction(this,FormAction.BTN_OK_CANCEL), BorderLayout.SOUTH);
		
	}

	public void performCancel() {
		close();
		
	}

	public void performOk() {
		List files = m_fileList.getData();
		if (files==null || files.size()==0) {
			m_article.setFile(null);
		} else {
			FileListMessage flm = new FileListMessage();
			flm.setFiles(files);
			m_article.setFile(flm);
		}
		if (m_article.getFile()!=null) {
			m_article.getFile().setOptions(m_binOptions.getOptions());
		}
		close();
		
	}

	//Not implemented
	public void performApply() {
	}
	public void performUndo() {
	}
	
	private void close() {
		SwingUtilities.getWindowAncestor(this).dispose();
	}

	public void stateChanged(ChangeEvent e) {
		if (e.getSource() instanceof CoffeeBean && m_fileList.getData()!=null) {
			FileListMessage fm = new FileListMessage();
			fm.setOptions((Options) e.getSource());
			fm.setFiles(m_fileList.getData());
			Article a = new Article();
			a.setFile(fm);
			fireCoffeeEvent(m_stats, CoffeeEvent.CALCULATE_POSTS, a);
		}
	};
	
	public int getSelectEvent(int messageType) {
    	return CoffeeEvent.NONE;
	}
}
