/* 	ExtMutableList.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Sept 4, 2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		Made functions public for delegate JList to set proportions of that list
 * 										Fixed bug with removing items from the list		
*/
package org.brains2b.coffee.gui.list;

import java.awt.BorderLayout;
import java.awt.dnd.DropTarget;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.brains2b.swingext.action.ExtActionMap;

public class ExtMutableList extends JPanel implements ActionListener {
	
	class ItemSelector implements ListSelectionListener {
	
	    public void valueChanged(ListSelectionEvent e) {
	        if (!e.getValueIsAdjusting() && e.getFirstIndex()>=0) {
	            getActionMap().get("ExtMutableList.Edit").setEnabled(true);
	            getActionMap().get("ExtMutableList.Remove").setEnabled(true);
	        }
	    }
	    
	}
	
	public final static int ACTION_ADD=1;
	public final static int ACTION_REMOVE=2;
	public final static int ACTION_EDIT=4;
	
	private JList m_list;
	private int m_availableActions;
	
	public ExtMutableList() {
		this(ACTION_ADD+ACTION_REMOVE+ACTION_EDIT);
	}
	
	public ExtMutableList(int availableActions) {
		super();
		m_availableActions =availableActions;
		init();
	}
	
	public void setFixedCellWidth(int width) {
		m_list.setFixedCellWidth(width);
	}
	
	public void setFixedCellHeight(int height) {
		m_list.setFixedCellHeight(height);
	}
	
	public void setVisibleRowCount(int visibleRowCount) {
		m_list.setVisibleRowCount(visibleRowCount);
	}
	
	public List getData() {
		//TODO should not use member directly
		return ((MutableListModel) m_list.getModel()).m_data;
	}
	
	public void setData(List l) {
		((MutableListModel) m_list.getModel()).setData(l);
	}
	
	public void clearData() {
		//TODO should not use member directly
		if (((MutableListModel) m_list.getModel()).m_data!=null) {
			((MutableListModel) m_list.getModel()).m_data.clear();
		}
	}
	
	public void setData(Object[] array) {
		setData(Arrays.asList(array));
	}
	
	public void addItem(Object o) {
		((MutableListModel) m_list.getModel()).addData(o);
	}

	public void replaceItem(Object o, int index) {
		((MutableListModel) m_list.getModel()).m_data.set(index, o);
		((MutableListModel) m_list.getModel()).fireContentsChanged(m_list.getModel(), index, index);
	}
	
	public void removeItem(int index) {
		((MutableListModel) m_list.getModel()).removeData(index);
	}
	
	public void setCellRenderer(ListCellRenderer cellRenderer) {
		m_list.setCellRenderer(cellRenderer);
	}
	
	public Object getSelectedValue() {
		return m_list.getSelectedValue();
	}
	
	public int getSelectedIndex() {
		return m_list.getSelectedIndex();
	}
	
	
	
	public void actionPerformed(ActionEvent e) {
		ActionEvent ne = new ActionEvent(this,e.getID(),e.getActionCommand());
		ActionListener[] listeners = (ActionListener[]) getListeners(ActionListener.class);
		for (int i=0;i<listeners.length;i++) {
			listeners[i].actionPerformed(ne);
		}
	}
	
	public void addListSelectionListener(ListSelectionListener listener) {
		m_list.addListSelectionListener(listener);
	}
	
	public void removeListSelectionListener(ListSelectionListener listener) {
		m_list.removeListSelectionListener(listener);
	}
	
	public void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}
	
	public void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}
	
	private void init() {
		ResourceBundle rs = ResourceBundle.getBundle("org.brains2b.coffee.resource.coffee");
		setLayout(new BorderLayout()); 
        m_list= new JList(new MutableListModel());
        m_list.addListSelectionListener(new ItemSelector());
        add(new JScrollPane(m_list),BorderLayout.CENTER);
        
        ExtActionMap map = new ExtActionMap(this);
        map.addAction("ExtMutableList.Add",rs.getString("ExtMutableList.Add"), rs.getString("ExtMutableList.Add.Tooltip"),
                "/org/brains2b/coffee/resource/Plus16.gif", "+", '+');
        map.addAction("ExtMutableList.Edit",rs.getString("ExtMutableList.Edit"), rs.getString("ExtMutableList.Edit.Tooltip"),
                "/org/brains2b/coffee/resource/Edit16.gif",  "+", '+');
        map.addAction("ExtMutableList.Remove",rs.getString("ExtMutableList.Remove"), rs.getString("ExtMutableList.Remove.Tooltip"),
                "/org/brains2b/coffee/resource/Minus16.gif","+", '+');
        setActionMap(map);
                
        JToolBar actions= new JToolBar(SwingConstants.VERTICAL);
        actions.setFloatable(false);
        if ((m_availableActions & ACTION_ADD)==ACTION_ADD)
        actions.add(getActionMap().get("ExtMutableList.Add"));
        if ((m_availableActions & ACTION_EDIT)==ACTION_EDIT)
        actions.add(getActionMap().get("ExtMutableList.Edit"));
        if ((m_availableActions & ACTION_REMOVE)==ACTION_REMOVE)
        actions.add(getActionMap().get("ExtMutableList.Remove"));
        
        //Wait for selection
        map.get("ExtMutableList.Edit").setEnabled(false);
        map.get("ExtMutableList.Remove").setEnabled(false);
        
        add(actions,BorderLayout.EAST);
	}
	
	public synchronized void setDropTarget(DropTarget dt) {
		super.setDropTarget(dt);
		m_list.setDropTarget(dt);
	}

	public Object getPrototypeCellValue() {
		return m_list.getPrototypeCellValue();
	}

	public void setPrototypeCellValue(Object prototypeCellValue) {
		m_list.setPrototypeCellValue(prototypeCellValue);
	}
	
	
}
