/* 	ArticleTable.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 25, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.gui.table;

import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;

/** Implementation of JTable for showing Articles in a JTree according 
 *  to references supplied 
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [Feb 25, 2008]
 * @see ArticleTreeTableModel
 * @see ArticleTreeCellRenderer
 */
public class ArticleTable extends JTable {

	class MouseHandler extends MouseAdapter {
	    public void mouseClicked(MouseEvent e) {
	        if (e.getSource() instanceof JTableHeader) {
	        	JTableHeader header=(JTableHeader) e.getSource();
	        	int idx=header.columnAtPoint(e.getPoint());
	        	((ArticleTreeTableModel) getModel()).sort(idx,false);
	        }
	       	super.mouseClicked(e);
	    }
	    
	    public void mousePressed(MouseEvent e) {
	        if (e.isPopupTrigger()) {
	        	popup(e);
	        }
	        super.mousePressed(e);
	    }
	    
	    public void mouseReleased(MouseEvent e) {
	        if (e.isPopupTrigger()) {
	        	popup(e);
	        }
	        super.mouseReleased(e);
	    }
	    
	    void popup(MouseEvent e) {
	        if (e.getSource() instanceof JTable) {
	            JTable t= (JTable)e.getSource();
	            int[] rows=t.getSelectedRows();
	            
	            if (rows==null || rows.length<=1) {
	                int row = t.rowAtPoint(e.getPoint());
	            	t.getSelectionModel().setSelectionInterval(row, row);
	            }
	            getPopMenu().show(t,e.getPoint().x,e.getPoint().y);
	        }
	    }
	}
	
	private JPopupMenu m_popMenu;

	public ArticleTable() {
		super();
		init();
	}
	
	public int getEditingRow() {
        return getColumnClass(editingColumn)==ArticleNode.class? -1 : editingRow;   
    }
	
	private void init() {
		ArticleTreeTableModel articleModel=new ArticleTreeTableModel();
        ArticleTreeCellRenderer articleRender = new ArticleTreeCellRenderer(articleModel,this);
        articleModel.setTree(articleRender);
        setDefaultEditor(ArticleNode.class, new ArticleTreeEditor(articleRender));
        setDefaultRenderer(ArticleNode.class, articleRender);
    	setDefaultRenderer(String.class, new ArticleTableCellRenderer());
        TreeSelectionAdapter selModel = new TreeSelectionAdapter(getSelectionModel());
       	articleRender.setSelectionModel(selModel);
        setModel(articleModel);
       	setColumnModel(new ArticleColumnModel());
        setShowGrid(false);
        setIntercellSpacing(new Dimension(0,0));
        articleRender.setRowHeight(getRowHeight());
        MouseHandler mh=new MouseHandler();
       	getTableHeader().addMouseListener(mh);
       	addMouseListener(mh);
	}

	private JPopupMenu getPopMenu() {
	    if (m_popMenu==null) {
	    	Object[] keys = getActionMap().keys();
	    	Arrays.sort(keys);
	    	m_popMenu=new JPopupMenu("artice list");
	    	for (int i=0;i<keys.length;i++) {
	    		m_popMenu.add(new JMenuItem(getActionMap().get(keys[i])));
	    	}
	    }
	    return m_popMenu;
	}
	
	/**
	 * get the ArticleTreeTableModel for this table
	 * <p>Just a convience method for getting <code>(ArticleTreeTableModel) getModel()</code> 
	 * @return {@link ArticleTreeTableModel}
	 */
	public ArticleTreeTableModel getArticleModel() {
		return (ArticleTreeTableModel) getModel();
	}
}
