/* 	Article.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Write newsgroup to XML output when it is not null		
*/
package org.brains2b.coffee.message;

import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import org.brains2b.coffee.message.local.FileMessage;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.XmlHelper;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [25-02-2008]
 */
public class Article implements NNTPMessage, Cloneable {
	private int m_headerSet=0;
    private int m_number;
    private String m_id;
    private String[] m_parts;
    private int m_totalParts=1;
    private int m_available=-1;
    private int m_part;
    private boolean m_new;
    private ArticleBody m_body;
    private ArticleHeader m_header;
    private boolean m_bodyStart;
    private transient String m_newsGroup;
    private FileMessage m_file;
    
    /**
     * 
     */
    public Article() {
        super();
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addLine(java.lang.String)
     */
    public void addLine(String s) {
        if (isHeaderSet()) {
	        if (m_headerSet==220) {
	            if (s.equals("\r\n")) {
                    m_bodyStart=true;
                }
	            if(!m_bodyStart) {
	            	m_header.addLine(s);
	            } else {
	                m_body.addLine(s);
	            }
	        } else {
	            m_body.addLine(s);
	        }
        } else {
            StringTokenizer st=new StringTokenizer(s,"\t");
            int i=0;
            setNew(true);
            if (m_header==null) {
            	m_header=new ArticleHeader();
            }
            while (st.hasMoreTokens()) {
                String t = st.nextToken();
                switch (i++) {
                    case 0:
                	    m_number=Integer.parseInt(t);
                	    break;
                	case 1:
                	    m_header.addHeaderValue("Subject",t);
                	    break;
                	case 2:
                	    m_header.addHeaderValue("From",t);
                	    break;
                	case 3:
                	    m_header.addHeaderValue("Date",t);
                	    break;
                	case 4:
                	    m_id=t;
                	    m_id=m_id.substring(1,m_id.length()-1);
                	    break;
                	case 5:
                        if (t.startsWith("<")) {
                            m_header.addHeaderValue("References",t);
                            break;
                        } else {
                            i++;
                        }
                	case 6:
                	    m_header.addHeaderValue("Bytes", t);
                	    break;
                	case 7:
                        m_header.addHeaderValue("Lines",t);
                        break;
                	default:
                	    if (t.length()>0 && t.indexOf(':')==-1 && Character.isDigit(t.charAt(0))) {
                	    	m_header.addHeaderValue("Lines",t);
                	    } 
                	    break;
                	
                	    
                	
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#addHeader(int, java.lang.String)
     */
    public void addHeader(int id, String s) {
        s=s.trim();
        switch (id) {
        	case 220:
        	    m_header=new ArticleHeader();
        	    m_bodyStart=false;
        	case 222:
        	    m_body=new ArticleBody();
        	case 223:
        	    int idx=s.indexOf(' ');
		        if (idx>0) {
		            m_number=Integer.parseInt(s.substring(0,idx).trim());
		        }
		        int last=idx+1;
		        idx=s.indexOf(' ',last);
		        if (idx>0) {
		            String messId=s.substring(last,idx).trim();
		            if (messId.charAt(0)=='<') {
                        last++;
                    }
		            if (messId.charAt(messId.length()-1)=='>') {
                        idx=messId.length()-2;
                    }
		            m_id=s.substring(last,idx+last);
		        }
		    	break;    
		    }
		m_headerSet=id;
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#isHeaderSet()
     */
    public boolean isHeaderSet() {
        return m_headerSet>0;
    }

    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#endMessage()
     */
    public void endMessage() {
    }
    
    public int getType() {
        return m_headerSet;
    }

    public void writeList(SAXWriter sw) throws SAXException {
        AttributesImpl atts=new AttributesImpl();
        atts.addAttribute("","","id","CDATA",XmlHelper.encode(m_id));
        atts.addAttribute("","","number","CDATA",String.valueOf(m_number));
        if (getNewsGroup()!=null) {
        	atts.addAttribute("","","newsgroup","CDATA",getNewsGroup());
        }
        if (m_totalParts>1) {
            atts.addAttribute("","","parts","CDATA",String.valueOf(m_totalParts)); 
        }
        sw.startElement("article",atts);
        if (m_header!=null) {
            m_header.writeHeader(sw);
        }
        if (m_parts!=null) {
            for (int i=0;i<m_parts.length && m_parts[i]!=null ;i++) {
                AttributesImpl attp=new AttributesImpl();
                attp.addAttribute("","","seq","CDATA",String.valueOf(i+1));
                attp.addAttribute("","","id","CDATA",XmlHelper.encode(m_parts[i]));
                sw.insertElement("part", attp);
            }
        }
        sw.endElement("article");
    }
    public String getId() {
        return m_id;
    }
    public void setId(String id) {
        m_id = id;
    }
    public int getNumber() {
        return m_number;
    }
    public void setNumber(int number) {
        m_number = number;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#writeMessage(org.brains2b.saxo.SAXWriter)
     */
    public void writeMessage(SAXWriter sw) throws SAXException {
       AttributesImpl atts=new AttributesImpl();
       atts.addAttribute(null,null,"id","CDATA",XmlHelper.encode(m_id));
       atts.addAttribute(null,null,"number","CDATA",String.valueOf(m_number));
       sw.startElement("article",atts);
       if (m_header!=null) {
           m_header.writeMessage(sw);
       }
       if (m_body!=null) {
           m_body.writeMessage(sw);
       }
       sw.endElement("body");
    }
    public ArticleBody getBody() {
        return m_body;
    }
    public void setBody(ArticleBody body) {
        m_body = body;
    }
    
    /* (non-Javadoc)
     * @see org.brains2b.coffee.message.NNTPMessage#hasContent()
     */
    public boolean hasContent() {
        return m_headerSet!=223;
    }
    public ArticleHeader getHeader() {
        return m_header;
    }
    public void setHeader(ArticleHeader header) {
        m_header = header;
    }
    
    public int getSize() {
       	if (m_header!=null) {
       	    return m_header.getSize();
       	} else {
       	    return 0;
       	}
    }
    public boolean isNew() {
        return m_new;
    }
    public void setNew(boolean new1) {
        m_new = new1;
    }
    
    public String getSubject() {
        if (m_header!=null) {
            return m_header.getSubject();
        } else {
            return null;
        }
    }
    public void setSubject(String subject) {
       if (m_header==null) {
           m_header = new ArticleHeader();
       }
       m_header.setSubject(subject);
    }
    public int getTotalParts() {
        return m_totalParts;
    }
    
    public void setAvailable(boolean b) {
    	if (!b) m_available=0;
    }
    
    public boolean isAvailable() {
    	if (m_available==-1) {
    		return getAvailableParts()==1+(m_parts==null?0:m_parts.length);
    	}
    	return m_available!=0;
    }
    
    private int getAvailableParts() {
        if (m_available==-1) {
            m_available=1;
            if (m_parts!=null) {
                for (int i=0;i<m_parts.length;i++) {
                    if (m_parts[i]!=null) {
                        m_available++;
                    }
                    
                }
            }
        }
        return m_available;
    }
    
    public void setTotalParts(int totalParts) {
        m_totalParts = totalParts;
        resize(totalParts-1);
    }
    
    public void addPart(int i, String s) {
        if (i<0) {
            i=0;
        }
        resize(i);
        m_parts[i]=s;
    }
    public List getParts() {
        if (m_parts==null) {
            return null;
        }
        return Arrays.asList(m_parts);
    }
    
    public boolean equals(Object o) {
        if (o instanceof Article && getId().equals( ((Article) o).getId())) {
            return true;
        } else {
            return false;
        }
    }
    public String getNewsGroup() {
        return m_newsGroup;
    }
    public void setNewsGroup(String newsGroup) {
        m_newsGroup = newsGroup;
    }
    
    public FileMessage getFile() {
        return m_file;
    }
    public void setFile(FileMessage file) {
        m_file = file;
    }
    public void setParts(List parts) {
        if (parts!=null) {
            parts.toArray(m_parts);
        }
    }

    public int getPart() {
        return m_part;
    }

    public void setPart(int part) {
        m_part = part;
    }
    
    private void resize(int size) {
        if (m_parts==null) {
            m_parts=new String[size];
        } else if (m_parts.length<size) {
            String[] parts= new String[size];
            System.arraycopy(m_parts, 0, parts, 0, m_parts.length);
        }
    }
    
    public String toString() {
    	String s = getSubject()==null?getId():getSubject();
    	if (getTotalParts()>1) {
    		s+= getPart()+" / " + getTotalParts();
    	}
    	return s;
    }
    
    void clearHeader() {
    	m_headerSet=0;
    }
    
    public Object clone() {
    	if (m_id!=null) {
    		throw new RuntimeException("You can only clone new messages");
    	}
    	Article a = new Article();
    	a.setHeader((ArticleHeader) getHeader().clone());
    	if (getBody()!=null) {
    		ArticleBody b = new ArticleBody();
    		b.addLine(getBody().getContent());
    		a.setBody(b);
    	}
    	return a;
    } 
}
