/* 	NewsGroupImpl.java
 *	Copyright (c) 2005-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Uses Newsgroup Interface	
*/
package org.brains2b.coffee.message;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/** Default implementation of NewsGroup
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [25-02-2008]
 */
public class NewsGroupImpl implements NewsGroup, NNTPMessage{

    private boolean m_headerSet;
    private boolean m_allowPost;
    private int m_type;
    private int m_first;
    private int m_last;
    private int m_estimate;
    private String m_name;
    private Date m_lastUpdated;
    protected List m_articleList;
    /**
     * 
     */
    public NewsGroupImpl() {
        super();
        m_type = 224;
    }

    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#isAllowPost()
	 */
    public boolean isAllowPost() {
        return m_allowPost;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setAllowPost(boolean)
	 */
    public void setAllowPost(boolean allowPost) {
        m_allowPost = allowPost;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getFirst()
	 */
    public int getFirst() {
        return m_first;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setFirst(int)
	 */
    public void setFirst(int first) {
        m_first = first;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getLast()
	 */
    public int getLast() {
        return m_last;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setLast(int)
	 */
    public void setLast(int last) {
        m_last = last;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getLastUpdated()
	 */
    public Date getLastUpdated() {
        return m_lastUpdated;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setLastUpdated(java.util.Date)
	 */
    public void setLastUpdated(Date lastUpdated) {
        if (lastUpdated==null && m_lastUpdated==null) {
            m_lastUpdated=new Date();
        } else if (lastUpdated!=null) {
            m_lastUpdated = lastUpdated;
        }
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getName()
	 */
    public String getName() {
        return m_name;
    }
    
    public String getId() {
        return m_name;
    }
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setName(java.lang.String)
	 */
    public void setName(String name) {
        m_name = name;
    }
    public void addHeader(int id, String s) {
        m_type=id;
        if (m_type==211) {
	        s=s.trim();
	        int idx=s.indexOf(' ');
	        if (idx>0) {
	        	setEstimate(Integer.parseInt(s.substring(0,idx).trim()));
	        }
	        int last=idx+1;
	        idx=s.indexOf(' ',last);
	        if (idx>0) {
	        	setFirst(Integer.parseInt(s.substring(last,idx).trim()));
	        }
	        last=idx+1;
	        idx=s.indexOf(' ',last);
	        if (idx>0) {
	        	setLast(Integer.parseInt(s.substring(last,idx).trim()));
	        }
	        last=idx+1;
	        setName(s.substring(last).trim());
	        setLastUpdated(new Date());
        }
		m_headerSet=true;
    }
    public void addLine(String s) {
        if (!isHeaderSet()) {
        	s=s.trim();
	        int idx=s.indexOf(' ');
	        if (idx>0) {
	        	setName(s.substring(0,idx));
	        }
	        int last=idx+1;
	        idx=s.indexOf(' ',last);
	        if (idx>0) {
	        	setLast(Integer.parseInt(s.substring(last,idx).trim()));
	        }
	        last=idx+1;
	        idx=s.indexOf(' ',last);
	        if (idx>0) {
	        	setFirst(Integer.parseInt(s.substring(last,idx).trim()));
	        }
	        last=idx+1;
	        setAllowPost("y".equalsIgnoreCase(s.substring(last).trim()));
	        setLastUpdated(new Date());
	        setEstimate(getLast()-getFirst());
        } else if (m_type==224) {
            Article a=new Article();
            a.addLine(s);
            addArticle(a);
        }

    }
    public void endMessage() {
    }
    public boolean isHeaderSet() {
        return m_headerSet;
    }
    
    /*private void processString(String s) {
        
            
    }*/
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getEstimate()
	 */
    public int getEstimate() {
        return m_estimate;
    }
    
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setEstimate(int)
	 */
    public void setEstimate(int estimate) {
        m_estimate = estimate;
    }
    
    /*public void writeStartElement(SAXWriter sw) throws SAXException {
        sw.startElement("newsgroup",getAttributes());
    }*/
    
    public void writeList(SAXWriter sw) throws SAXException {
        sw.insertElement("newsgroup",getAttributes());
    }
    
    protected Attributes getAttributes() {
        AttributesImpl atts=new AttributesImpl();
        atts.addAttribute("","","name","CDATA",XmlHelper.encode(getName()));
        atts.addAttribute("","","first","CDATA",String.valueOf(getFirst()));
        atts.addAttribute("","","last","CDATA",String.valueOf(getLast()));
        atts.addAttribute("","","allowPost","CDATA",String.valueOf(isAllowPost()));
        if (getLastUpdated()!=null) {
        	atts.addAttribute("","","lastUpdated","CDATA",DateHelper.isoFromDate(getLastUpdated()));
        }
        return atts;
    }
    
    public void writeMessage(SAXWriter sw) throws SAXException {
        sw.startElement("newsgroup",getAttributes());
        if (m_articleList!=null) {
            Iterator it=m_articleList.iterator();
            while (it.hasNext()) {
                Article a= (Article) it.next();
                if (a.getNumber()>getFirst() && (a.isAvailable() || a.getTotalParts()>1)) {
                    a.writeList(sw);
                }
            }
        }
     	sw.endElement("newsgroup");
    }
    
    /*public void writeEndElement(SAXWriter sw) throws SAXException {
       sw.endElement("newsgroup"); 
    }*/
    
    public int getType() {
        return m_type;
    }
    
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#addArticle(org.brains2b.coffee.message.Article)
	 */
    public void addArticle(Article a) {
        if (m_articleList==null) {
            m_articleList=new ArrayList();
        }
        m_articleList.add(a);
    }
    
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#removeArticle(org.brains2b.coffee.message.Article)
	 */
    public void removeArticle(Article a) {
        if (m_articleList!=null) {
        	m_articleList.remove(a);
        }
    }
    
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#getArticleList()
	 */
    public List getArticleList() {
        return m_articleList;
    }
 
    /* (non-Javadoc)
	 * @see org.brains2b.coffee.message.Newsgroup#setArticleList(java.util.List)
	 */
    public void setArticleList(List articleList) {
        m_articleList = articleList;
    }
    
    public boolean hasContent() {
        return m_type!=211;
    }
    
    public int getSize() {
        return (m_estimate==0?m_last-m_first:m_estimate);
    }
    
}
