/* 	PostArticle.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 14-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message;

import java.security.SecureRandom;
import java.util.Random;

import sun.misc.BASE64Encoder;

public class PostArticle {

    private static Random m_rnd;
    private static BASE64Encoder m_encoder = new BASE64Encoder();
    
    public final static String create(Article a) {
        StringBuffer sb =new StringBuffer();
        createHeader(sb,a.getHeader());
        createBody(sb, a.getBody());
        return sb.toString();
    }
    
    private final static void createHeader(StringBuffer sb, ArticleHeader header) {
        sb.append("From: ").append(header.getAuthor()).append("\r\n");
        sb.append("Subject: ").append(header.getSubject()).append("\r\n");
        if ( header.getHeaderValue("References") != null) {
            sb.append("References: ").append(header.getHeaderValue("References")).append("\r\n");
        }
        sb.append("Newsgroups: ").append(header.getHeaderValue("Newsgroups")).append("\r\n");
        //sb.append("Message-ID: ").append(createID(header.getAuthor())).append("\n");
    }
    
    private final static void createBody(StringBuffer sb, ArticleBody body) {
        sb.append("\r\n");
        sb.append(body.getContent());
        sb.append("\r\n.\r\n");
    }
    
    private final static String createID(String author) {
        
        StringBuffer id= new StringBuffer();
        byte[] b = new byte[16];
        if (m_rnd==null) {
            try {
                m_rnd= SecureRandom.getInstance("SHA1PRNG");
            } catch (Exception ex) {
                m_rnd = new SecureRandom();
            }
        }
        m_rnd.nextBytes(b);
        id.append("<");
        id.append(m_encoder.encode(b));
        int idx=author.indexOf("@");
        
        int last=author.length();
        for (int i=idx;i<author.length();i++) {
            if (author.charAt(i)==' ' || author.charAt(i)=='(' || author.charAt(i)=='<') last=idx;
        }
        id.append(author.substring(idx,last));
        id.append(">");
        return id.toString();
    }
}
