/* 	CoffeeMenu.java
 *	Copyright (c) 2007, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 17-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.message.local;

import java.util.ArrayList;
import java.util.Iterator;

import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.saxo.SAXWriter;
import org.brains2b.util.StringHelper;
import org.xml.sax.SAXException;

public class CoffeeMenu implements NNTPMessage{

    private String m_name;
    private Class m_clazz;
    private ArrayList m_list;
    
    
    public CoffeeMenu(String name,Class c) {
        super();
        m_name=name;
        if (!CoffeeBean.class.isAssignableFrom(c)) {
            throw new IllegalArgumentException("Lists can only be build from CoffeeBeans");
        }
        m_clazz=c;
    }

    public void add(NNTPMessage message) {
        if (message.getClass()!=m_clazz) {
            throw new IllegalArgumentException("Adding wrong type of message to list of " + m_name);
        }
        if (m_list==null) {
            m_list=new ArrayList();
        }
        m_list.add(message);
    }
    
    public void remove(NNTPMessage message) {
        if (m_list!=null) {
            m_list.remove(message);
        }
    }
    
    public NNTPMessage[] getArray() {
        if (m_list==null) return new NNTPMessage[0];
        NNTPMessage[] list= new NNTPMessage[m_list.size()];
        m_list.toArray(list);
        return list;
    }
    
    public CoffeeBean createBean() {
        try {
            CoffeeBean cb=(CoffeeBean) m_clazz.newInstance();
            add(cb);
            return cb;
        } catch (Exception ex) {
            return null;
        }
    }
    
    public String getId() {
        return m_name;
    }


    public int getSize() {
       return m_list==null?0:m_list.size();
    }


    public int getType() {
        return 0;
    }


    public boolean hasContent() {
        return m_list!=null && !m_list.isEmpty();
    }


    public boolean isHeaderSet() {
       return false;
    }


    public void writeMessage(SAXWriter sw) throws SAXException {
        sw.startElement(makeMultiple(m_name), null);
        if (m_list!=null)
            for (Iterator it = m_list.iterator();it.hasNext();) {
                NNTPMessage m= (NNTPMessage) it.next();
                m.writeMessage(sw);
            }
        sw.endElement(makeMultiple(m_name));
    }
    
    //Not implemented because it is never supplied remote
    public void addHeader(int id, String s) {
    }


    //Not implemented because it is never supplied remote
    public void addLine(String s) {
    }

    //Not implemented because it is never supplied remote
    public void endMessage() {
    }
    
    public static final String makeMultiple(String name) {
    	if (StringHelper.isEmpty(name)) throw new NullPointerException("Name for configuration file cannot be null");
    	final int lastChar =name.length()-1;
    	if (name.charAt(lastChar) == 'y') {
    		return name.substring(0, lastChar) + "ies"; 
    	} else if (name.charAt(lastChar) == 's') {
    		return name.substring(0, lastChar) + "z";
    	} else {
    		return name + "s";
    	}
    }
}
