/* 	MessageParser.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 16-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 25, 2008	dennis@brains2b.nl		Use String.valueOf instead of new String()		
*/
package org.brains2b.coffee.nntp;

import java.util.Vector;

import org.brains2b.coffee.message.MessageFactory;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.monitor.ProgressMonitor;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class MessageParser {
    private Vector m_monitors;
    private NNTPMessage m_message;
    private NNTPMessage m_invokedOn;
    private boolean m_done;
	private char[] buffer= new char[2048];
	private int i=0;
	private int cnt=0;
    /**
     * 
     */
    public MessageParser() {
        super();
        
    }
    
    public void addChar(char c) {
        if (i<buffer.length) {
            buffer[i++]=c;
        }
        if (c==0xa) {
     		setContent(buffer,0,i);
     		i=0;
     	}
    }
    
    private void setContent(char[] content, int begin, int end) {
        if (m_message==null) {
            int type=Integer.parseInt(String.valueOf(content,0,3));
   			if (m_invokedOn==null) {
            	m_message=MessageFactory.createMessage(type);
   			} else {
   			    //TODO Should check for valid valueson type
   			    m_message=m_invokedOn;
   			    if (cnt++>100 && cnt%100==0) {
                	advance(0,cnt,m_message.getSize());
            	}
   			}
   			m_message.addHeader(type,String.valueOf(content,3,end-3));
   			m_done=!m_message.hasContent();
   			if (!m_done && m_invokedOn!=null) {
   			    m_message=m_invokedOn;
   			    
   			}
        } else if (content[0]=='.' && end<=3) {
        	m_done=true;
        } else {
            m_message.addLine(String.valueOf(content,begin,end));
            if (cnt++>100 && cnt%100==0) {
                advance(0,cnt,m_message.getSize());
            }
        }
    }

    public void start() {
        m_message=null;
        m_done=false;
        i=0;
        cnt=0;
    }
    
    public void end() {
        m_message.endMessage();
        m_message=null;
    }
    
    public NNTPMessage getMessage() {
        return m_message;
    }
    
    public boolean isDone() {
        return m_done;
    }
    
    public void addProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors==null) {
            m_monitors=new Vector();
        }
        m_monitors.add(monitor);
    }
    
    public void removeProgressMonitor(ProgressMonitor monitor) {
        if (m_monitors!=null) {
            m_monitors.remove(monitor);
        }
    }
    
    public void advance(int start, int value, int end) {
        if (m_monitors!=null && m_message!=null && m_message.hasContent()) {
            for (int i=0;i<m_monitors.size();i++) {
                ((ProgressMonitor) m_monitors.get(i)).advance(start,value,end);
            }
        }
    }
    
    public void setMessage(NNTPMessage message) {
        m_invokedOn = message;
    }
}
