/* 	NNTPConnector.java
 *	Copyright (c) 2005, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 16-jul-2005
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * May 8, 2007	dennis@brains2b.nl      Read correct content type ISO-8559-1 from outset 		
*/
package org.brains2b.coffee.nntp;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.SocketException;
import java.nio.charset.Charset;

import org.brains2b.coffee.message.InfoMessage;
import org.brains2b.coffee.message.NNTPMessage;

/** [Short description]
 *  [Long description]
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version [version] [lastrevisiondate]
 */
public class NNTPConnector {
	
    private Socket m_socket;
    private MessageParser m_parser;
    private String m_server;
    private String m_userName;
    private String m_passWord;
    private int m_port;
    private boolean m_available;
    
    /**
     * 
     */
    public NNTPConnector(String server, int port) {
        this (server,port,null,null);
    }
    
    /**
     * 
     */
    public NNTPConnector(String server, int port, String userName, String passWord) {
        super();
        m_parser=new MessageParser();
        m_server=server;
        m_port=port;
 		m_userName=userName;
 		m_passWord=passWord;
    }

    public NNTPMessage connect() throws IOException {
        m_socket=new Socket(m_server,m_port);
        m_socket.setKeepAlive(true);
        m_socket.setReuseAddress(true);
        InputStreamReader isr=new InputStreamReader(m_socket.getInputStream());
        m_parser.start();
        m_parser.setMessage(new InfoMessage());
        int c= isr.read();
        while (c!=-1) {
                
            m_parser.addChar((char)c);
            if (m_parser.isDone()){  
                break;
            }
            c= isr.read();
        }
        NNTPMessage result = m_parser.getMessage();
        m_parser.end();
        NNTPMessage msg = null;
        if (result.getType()!=400) {
        	if (m_userName!=null) {
        		msg = invoke("AUTHINFO USER",m_userName);
        		if (msg.getType()>400) return msg;
        	}
        	
        	if (m_passWord!=null) {
        		msg = invoke("AUTHINFO PASS",m_passWord);
        		if (msg.getType()>400) return msg;
        	}
        	
            msg =invoke("MODE","READER");
            return msg;
        } else  {
            return result;
        }
            
        
    }
    
    public NNTPMessage invoke(String action) throws IOException {
        return invoke(action,null);
    }
    public NNTPMessage invoke(String action, String param) throws IOException {
        return invokeOn(null,action,param);
    }
    public synchronized NNTPMessage invokeOn(NNTPMessage message, String action, String param) throws IOException {
        try {
           if (m_socket.isClosed()) {
               connect();
           }
	       BufferedReader isr=new BufferedReader(new InputStreamReader(m_socket.getInputStream(),Charset.forName("ISO-8859-1")));
	       OutputStreamWriter osw=new OutputStreamWriter(m_socket.getOutputStream(),Charset.forName("ISO-8859-1"));
	        
	       osw.write(action+(param!=null?" "+ param+"\n":"\n"));
	       osw.flush();
       	   m_parser.start();
       	   m_parser.setMessage(message);
            int c= isr.read();
            while (c!=-1) {
                
                m_parser.addChar((char)c);
                if (m_parser.isDone()){  
                    break;
                }
                c= isr.read();
            }
            
            NNTPMessage result = m_parser.getMessage();
            m_parser.end();
            return result;
        } catch (SocketException sex) {
            m_socket.close();
            throw sex;
        }
        
    }
    
    public NNTPMessage post(String message) throws IOException {
        try {
           if (m_socket.isClosed()) {
               connect();
           }
           NNTPMessage msg =invokeOn(null, "POST", null);
           if (msg.getType()!=340) {
               return msg;
           }
           InputStreamReader isr=new InputStreamReader(m_socket.getInputStream(),Charset.forName("ISO-8859-1"));
           OutputStreamWriter osw=new OutputStreamWriter(m_socket.getOutputStream(),Charset.forName("ISO-8859-1"));
           
           osw.write(message);
           osw.flush();
           m_parser.start();
           int c= isr.read();
           while (c!=-1) {
                m_parser.addChar((char)c);
                if (m_parser.isDone()){  
                    break;
                }
                c= isr.read();
            }
            
            NNTPMessage result = m_parser.getMessage();
            m_parser.end();
            return result;
        } catch (SocketException sex) {
            m_socket.close();
            throw sex;
        }
        
    
    }
    
    public void close() throws IOException {
        try {
            BufferedWriter bw=new BufferedWriter(new OutputStreamWriter(m_socket.getOutputStream()));
            bw.write("QUIT\n");
            bw.flush();
            bw.close();
        } catch (SocketException iex) {
            //No gracefull exit: Broken pipe
        } finally {
       	    m_socket.close();
        }
    }
    
    public MessageParser getParser() {
        return m_parser;
    }

    public boolean isAvailable() {
        return m_available;
    }

    public void setAvailable(boolean available) {
        m_available = available;
    }
}
