/* 	MimePlugin.java
 *	Copyright (c) 2007-2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 13-jan-2007
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * Feb 26, 2008	dennis@brains2b.nl		Implement open and close for Plugins		
*/
package org.brains2b.coffee.plugin;

import org.brains2b.coffee.file.FileConnector;

/**
 * Interface for defining MimePlugins
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.89.1 [26-02-2008]
 */
public interface MimePlugin {
    
	/**
	 * check if the plugin supports the particular eventID
	 * for this MimeType
	 * @param mime String, the name of the mime-type
	 * @param eventID, the CoffeeEvent id to support
	 * @return boolean, true if this plugin supports the action
	 * on the mime-type false otherwise
	 */
    public boolean isActionSupported(String mime, int eventID);
    
    /**
     * get short title on which this plugin is known
     * @return
     */
    public String getShortTitle();
    
    /**
     * open is called after instantiation and before using the plugin
     * @see MimeHandler
     */
    public void open();
    
    /**
     * close is called before the application is closed and with 
     * only the {@link FileConnector} available to handle events
     * @see MimeHandler
     */
    public void close();
}
