/* 	FilterNewsGroup.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-feb-2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.search;

import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.saxo.SAXWriter;
import org.xml.sax.SAXException;

/**
 * Implementation of NewsGroup to add articles only when they
 * match te values in the filters
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [25-02-2008]
 */
public class FilterInNewsGroup implements NewsGroup {
    
	private List m_filters;
    private NewsGroup m_group;

    /**
     * Constructor
     * @param group {@link NewsGroup} the group to search in
     * @param filters List, list of {@link MessageFilter}
     */
    public FilterInNewsGroup(NewsGroup group, List filters) {
        super();
    	m_filters=filters;
        m_group = group;
        
    }
    
    public void addArticle(Article a) {
		boolean add=false;
    	for (Iterator itf = m_filters.iterator();itf.hasNext();) {
    		MessageFilter mf = (MessageFilter) itf.next();
    		if (mf.getKey().equals("new")) {
    			add = a.isNew();	
    		} else { 
    			Object value =a.getHeader().getHeaderObject(mf.getKey());
    			add = add || mf.isValid(value);
    		}
    	}
       	if (add) m_group.addArticle(a);
   
    }

	public void addHeader(int id, String s) {
		m_group.addHeader(id, s);
	}

	public void addLine(String s) {
		if (m_group.getType()==224 || m_group.getType()==Subscription.SUBSCRIPTION) {
            Article a=new Article();
            a.addLine(s);
            addArticle(a);
        } else {
        	m_group.addLine(s);
        }
	}
	public void endMessage() {
		m_group.endMessage();
	}

	public List getArticleList() {
		return m_group.getArticleList();
	}

	public int getEstimate() {
		return m_group.getEstimate();
	}

	public int getFirst() {
		return m_group.getFirst();
	}

	public String getId() {
		return m_group.getId();
	}

	public int getLast() {
		return m_group.getLast();
	}

	public Date getLastUpdated() {
		return m_group.getLastUpdated();
	}

	public String getName() {
		return m_group.getName();
	}

	public int getSize() {
		return m_group.getSize();
	}

	public int getType() {
		return m_group.getType();
	}

	public boolean hasContent() {
		return m_group.hasContent();
	}

	public boolean isAllowPost() {
		return m_group.isAllowPost();
	}

	public boolean isHeaderSet() {
		return m_group.isHeaderSet();
	}

	public void removeArticle(Article a) {
		m_group.removeArticle(a);
	}

	public void setAllowPost(boolean allowPost) {
		m_group.setAllowPost(allowPost);
	}

	public void setArticleList(List articleList) {
		m_group.setArticleList(articleList);
	}

	public void setEstimate(int estimate) {
		m_group.setEstimate(estimate);
	}

	public void setFirst(int first) {
		m_group.setFirst(first);
	}

	public void setLast(int last) {
		m_group.setLast(last);
	}

	public void setLastUpdated(Date lastUpdated) {
		m_group.setLastUpdated(lastUpdated);
	}

	public void setName(String name) {
		m_group.setName(name);
	}

	public void writeMessage(SAXWriter sw) throws SAXException {
		m_group.writeMessage(sw);
	}
	
	public void writeList(SAXWriter sw) throws SAXException {
    	m_group.writeList(sw);
    }
    
}
