/* 	NewsSearchProcess.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: Feb 25, 2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]					[What]
 * 28-03-2008	dennis@brains2b.org		Fixed calculation of first and last for getArticles()
 * 09-04-2008	dennis@brains2b.org		findNewsGroup should also check previous queries		
 */
package org.brains2b.coffee.plugin.search;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;

import org.brains2b.coffee.TicketBroker;
import org.brains2b.coffee.event.CoffeeEvent;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.local.CoffeeMenu;
import org.brains2b.coffee.message.local.ServerDescriptor;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.coffee.plugin.CoffeeExchangeControl;
import org.brains2b.task.Task;
import org.brains2b.task.TaskEvent;
import org.brains2b.task.TaskListener;
import org.brains2b.util.StringHelper;

/** Main Processor for Queries
 * <p>Runs query messages by calling articles list 50.000 at a time 
 * for the newsgroups in this query one by one. FilterInNewsGroup is
 * used to filter in only the articles matching. All others will be ignored
 * to save memory.
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90.1 [09-04-2008]
 * @see FilterInNewsGroup
 */
public class NewsSearchProcess implements CoffeeExchangeControl, TaskListener {

	private final static int MAX = 50000;
	
	private boolean m_finished =true;
	private int m_taskSpawned =0;
	private Date m_thisDate;
	private ServerDescriptor m_server;
	private String m_findName;
	private QueryMessage m_current;
	private ArrayList m_listeners;
	private List m_queries = Collections.synchronizedList(new ArrayList());
	
	/**
	 * Constructor
	 *
	 */
	public NewsSearchProcess() {
		super();
	}
	
	public TaskListener getTaskListener() {
		return this;
	}

	public Stack getMessages() {
		return null;
	}

	public int getResult() {
		return RESULT_OK;
	}

	public boolean isFinished() {
		return m_finished;
	}

	public void setFinished(boolean b) {
		m_finished = b;
	}

	public void run() {
		setFinished(false);
		m_thisDate = new Date();
		for (Iterator it = m_queries.iterator(); it.hasNext();) {
			synchronized (m_queries) {
				m_current = (QueryMessage) it.next();
				Calendar c= Calendar.getInstance();
				c.roll(Calendar.DAY_OF_YEAR,-m_current.getFrequency());
				if (m_current.getNewsgroups()!=null) {
					for (Iterator itN = m_current.getNewsgroups().iterator();itN.hasNext();) {
						Subscription ng = (Subscription) itN.next();
						if (m_current.isDirect() || ng.getLastUpdated().before(c.getTime())) {
							findServer(ng.getServer());
							FilterInNewsGroup fin = new FilterInNewsGroup((Subscription) ng.clone(),m_current.getFilters());
							fin.setFirst(ng.getLast());
							fin.setLast(ng.getLast()+MAX);
							m_taskSpawned++;
							TicketBroker.getBroker().broker(
								new CoffeeEvent(this,CoffeeEvent.GET_ARTICLE_LIST,fin)
							);
						}
					}
					cleanup();
					TaskEvent te = new TaskEvent(m_current,0,0,m_current.getNewsgroups().size(),m_current.getNewsgroups().size());
					te.setFinished(true);
					m_current.setDirect(false);
					fireTaskEvent(te);
				}
				while (m_taskSpawned>0) {
					try {
						Thread.sleep(1000);
					} catch (InterruptedException iex){}
				}
			}
			
			
		}
		setFinished(true);
	}

	public void setCurrentMessage(NNTPMessage message) {
		if (message instanceof NewsGroup) {
			NewsGroup ng = (NewsGroup) message;
			if (m_current.getArticles()==null) {
				m_current.setArticles(new ArrayList());
			} 
			if (ng.getArticleList()!=null) {
				for( Iterator it = ng.getArticleList().iterator();it.hasNext();) {
					Article a = (Article) it.next();
					a.setNewsGroup(ng.getName());
					if (!m_current.getArticles().contains(a)) {
						m_current.addArticle(a);
					}
				}
			}
			Subscription grp = findNewsGroup(m_current, ng.getName());
			
			findServer(grp.getServer());
			if (grp!=null && grp.getName().equals(ng.getName()) && ng.getLast()-ng.getFirst()<MAX) {
				grp.setLast(ng.getLast());
				if (grp.getLastUpdated().before(m_thisDate)) {
					grp.setFirst(ng.getFirst());
				}
				grp.setLastUpdated(m_thisDate);
				fireTaskEvent(new TaskEvent(m_current,0,0,m_current.getNewsgroups().indexOf(grp),m_current.getNewsgroups().size()));
			}
			if (grp!=null && ng.getLast()-ng.getFirst()>=MAX) {
				ng.setFirst(ng.getLast());
				ng.setLast(ng.getLast()+MAX);
				m_taskSpawned++;
				TicketBroker.getBroker().broker(
					new CoffeeEvent(this,CoffeeEvent.GET_ARTICLE_LIST,ng)
				);
			}
			ng =null;
			
		} else if ( message instanceof CoffeeMenu && message.getId().equals("server")) {
   			NNTPMessage[] servers = ((CoffeeMenu) message).getArray();
   			for (int i=0;i<servers.length;i++) {
   				if (m_findName.equals(servers[i].getId())) {
   					m_server = (ServerDescriptor) servers[i];
   					break;
   				}
   			}
		}
		m_taskSpawned--;
	}

	/**
	 * add a QueryMessage to this process
	 * <p>Works on both finished and running process
	 * @param qm {@link QueryMessage} to add
	 */
	public void addQueryMessage(QueryMessage qm) {
		m_queries.add(qm);
	}
	
	/**
	 * remove a QueryMessage from this process
	 * <p>Works on running process, but not if this
	 * is the current one running
	 * @param qm {@link QueryMessage} to add
	 */
	public void removeQueryMessage(QueryMessage qm) {
		m_queries.remove(qm);
	}

	public void taskProgressed(TaskEvent e) {
		// decrease taskSpawned only if there is no current message
		if (e.isFinished() && ((Task) e.getSource()).getResult()!=Task.RESULT_OK) {
			m_taskSpawned--;
		}
		
	}
	
	/**
	 * add a TaskListener to this process
	 * @param tl TaskListener
	 */
	public void addTaskListener(TaskListener tl) {
		if (m_listeners==null) {
			m_listeners=new ArrayList();
		}
		m_listeners.add(tl);
	}

	/**
	 * remove a TaskListener from this process
	 * @param tl TaskListener
	 */
	public void removeTaskListener(TaskListener tl) {
		if (m_listeners!=null) {
		    m_listeners.remove(tl);
		}
	}
	
	private Subscription findNewsGroup(QueryMessage qm, String name) {
		List l = qm.getNewsgroups();
		if (l==null || StringHelper.isEmpty(name)) return null;
		for (Iterator it = l.iterator();it.hasNext();) {
			Subscription ng = (Subscription) it.next();
			if (name.equals(ng.getName())) {
				return ng;
			}
		}
		// Might be from previous query
		int idx = m_queries.indexOf(qm);
		if (idx-1>=0) return findNewsGroup((QueryMessage) m_queries.get(idx-1), name);
		
		return null;
	}
	
	private void fireTaskEvent(TaskEvent e) {
	    if (m_listeners!=null) {
	        Iterator it=m_listeners.iterator();
	        while (it.hasNext()) {
	            ((TaskListener) it.next()).taskProgressed(e); 
	        }
	    }
	}
	
	private synchronized ServerDescriptor findServer(String name) {
		if (m_server!=null && name.equals(m_server.getId())) {
			return m_server;
		}
		m_server = TicketBroker.getBroker().getCurrentServer();
		if (m_server!=null && name.equals(m_server.getId())) {
			return m_server;
		}
		m_server = null;
		m_findName = name;
		TicketBroker.getBroker().broker(
			new CoffeeEvent(this,CoffeeEvent.GET_LIST,"server")		
		);
		while (m_server==null) {
			try {
				Thread.sleep(500);
			} catch (Exception ex) {}
		}
		TicketBroker.getBroker().broker(
			new CoffeeEvent(this,CoffeeEvent.CHANGE_SERVER,m_server)		
		);
		return m_server;
	}
	
	private void cleanup() {
		List l =m_current.getArticles();
		if (l==null) return;
		NewsGroup lastNg =null;
		for (Iterator it= l.iterator();it.hasNext();) {
			Article a = (Article) it.next();
			if (lastNg==null || (a.getNewsGroup()!=null && !a.getNewsGroup().equals(lastNg.getId()))) {
				lastNg = findNewsGroup(m_current, a.getNewsGroup());
			}
			if (lastNg!=null && a.getNumber()>lastNg.getFirst()) {
				it.remove();
			}
		}
	}
}
