/* 	QueryHandler.java
 *	Copyright (c) 2008, Brains2B.org
 *	
 *	Created by: Dennis Groenendijk
 *	Created on: 25-feb-2008
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that  * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.brains2b.coffee.plugin.search;

import org.brains2b.coffee.file.FileContentHandler;
import org.brains2b.coffee.message.Article;
import org.brains2b.coffee.message.ArticleHeader;
import org.brains2b.coffee.message.NNTPMessage;
import org.brains2b.coffee.message.NewsGroup;
import org.brains2b.coffee.message.filter.MessageFilter;
import org.brains2b.coffee.message.local.Subscription;
import org.brains2b.util.DateHelper;
import org.brains2b.util.XmlHelper;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/** ContentHandler for reading queries from file
 *
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.90 [Feb 25, 2008]
 */
public class QueryHandler extends FileContentHandler {

	private String m_key;
    private QueryMessage m_query;
    private NNTPMessage m_last;
    
    /**
     * Constructor
     *
     */
    public QueryHandler() {
        super();
        setDefaultDir("query");
    }
    
    public NNTPMessage getMessage() {
        return m_query;
    }

    public void endDocument() throws SAXException {
        
    }

    public void startDocument() throws SAXException {
        
    }

    public void characters(char[] ch, int start, int length)
            throws SAXException {
        if (m_last instanceof MessageFilter) {
    		((MessageFilter) m_last).setValue(XmlHelper.decode(String.valueOf(ch,start,length)));
        } else if (m_last instanceof Article && m_key!=null) {
        	((Article) m_last).getHeader().addHeaderValue(m_key, XmlHelper.decode(String.valueOf(ch,start,length)));
        }
    }

    public void endElement(String namespaceURI, String localName, String qName)
            throws SAXException {
		 if (qName.equals(m_key)) m_key = null;
    }

    public void startElement(String namespaceURI, String localName,
            String qName, Attributes atts) throws SAXException {
       if (qName.equals("querydef")) {
    	   QueryMessage msg = new QueryMessage();
           msg.setId(atts.getValue("id"));
           msg.setFrequency(atts.getValue("frequency")==null?1:Integer.parseInt(atts.getValue("frequency")));
           m_query = msg;
       } else if (qName.equals("query")) {
    	   MessageFilter filter =new MessageFilter();
           m_query.addFilter(filter);
           filter.setKey(atts.getValue("field"));
           filter.setNot("true".equals(atts.getValue("not")));
           filter.setOperator(atts.getValue("operation"));
           m_last = filter;
       } else if (qName.equals("newsgroup")) {
    	   NewsGroup grp = createNewsGroup(atts);
    	   m_query.addNewsGroup(grp);
    	   m_last = grp;
       } else if (qName.equals("article")) {
    	   Article a = createArticle(atts);
    	   a.setHeader(new ArticleHeader());
    	   m_query.addArticle(a);
    	   m_last =a ;
       } else if (qName.equals("part")) {
    	    ((Article)m_last).addPart(Integer.parseInt(atts.getValue("seq"))-1,atts.getValue("id"));
       } else if (!qName.equals("header")) {
    	   m_key = qName;
       }
	  
    }

    private NewsGroup createNewsGroup(Attributes atts) {
        Subscription group= new Subscription();
        group.setName(atts.getValue("name"));
        group.setFirst(Integer.parseInt(atts.getValue("first")));
        group.setLast(Integer.parseInt(atts.getValue("last")));
        group.setAllowPost("true".equals(atts.getValue("allowPost")));
        group.setServer(atts.getValue("server"));
        group.setLastUpdated(DateHelper.isoToDate(atts.getValue("lastUpdated")));
        return group;
    }

	private Article createArticle(Attributes atts) {
	    Article a=new Article();
	    a.setId(atts.getValue("id"));
	    a.setNewsGroup(atts.getValue("newsgroup"));
	    a.setNumber(Integer.parseInt(atts.getValue("number")));    
	    if (atts.getValue("parts")!=null) {
	        a.setTotalParts(Integer.parseInt(atts.getValue("parts")));
	    }
	    return a;
	}
}
